# Developer(s): Grigori Fursin

import cmind
import misc
import os


def page(st, params):

    url_prefix = st.config.get_option('server.baseUrlPath') + '/'

    name = params.get('name', [''])[0].lower()

    list_all = False

    if name != '':
        r = cmind.access({'action': 'load',
                          'automation': 'contributor,68eae17b590d4f8f',
                         'artifact': name})
        if r['return'] > 0 and r['return'] != 16:
            return r

        end_html = ''

        if r['return'] == 0:
            meta = r['meta']

            path = r['path']

            name = meta.get('name', meta.get('organization', ''))
            if name != '':
                st.markdown("#### " + name)

                x = ''
                for t in meta.get('trophies', []):
                    url = t.get('url', '')
                    if url != '':
                        x += '<a href="{}">&#127942</a>&nbsp;'.format(url)

                if x != '':
                    st.write('<h2>' + x + '</h2>', unsafe_allow_html=True)

                end_html = '''
                 <center>
                  <small><a href="{}"><i>Self link</i></a></small>
                 </center>
                 '''.format(misc.make_url(meta['uid'], action='contributors', md=False))

                org = meta.get('organization', '')
                if org != '':
                    st.markdown("* **Organization:** " + org)

                urls = meta.get('urls', [])

                url = meta.get('url', '')
                if url != '':
                    urls.append(url)

                if len(urls) > 0:
                    x = '* **Web page:** '
                    md = ''
                    if len(urls) > 1:
                        md = '* **Web pages:**\n'
                        x = '   * '

                    for u in urls:
                        md += x + '[{}]({})\n'.format(u, u)

                    st.markdown(md)

                ongoing = meta.get('ongoing', [])

                x = str(calculate_points(meta))
                y1 = ''
                y2 = ''
                if len(ongoing) > 0:
                    y1 = '*'
                    y2 = ' (ongoing)*'
                st.markdown("* **Points: {}{}{}**".format(y1, x, y2))
#                st.write('<h2>'+x+'</h2>', unsafe_allow_html = True)

                if len(ongoing) > 0:
                    x = "* **Ongoing challenges:**\n"

                    for t in ongoing:
                        if t != '':
                            x += "   - {}\n".format(misc.make_url(t,
                                                    action='challenges', key='tags'))

                    st.markdown(x)

                challenges = meta.get('challenges', [])
                if len(challenges) > 0:
                    md = "* **Contributions:**\n"

                    for c in sorted(challenges):
                        md += "  * {}\n".format(misc.make_url(c,
                                                action='challenges', key='tags'))

                    st.markdown(md)

                # Check if README
                md = ''

                readme = os.path.join(path, 'README.md')
                if os.path.isfile(readme):

                    r = cmind.utils.load_txt(readme)
                    if r['return'] > 0:
                        return r

                    md += r['string']

                st.markdown('---')
                st.markdown(md)

        else:
            st.markdown(
                '**Warning:** Contributor "{}" not found!'.format(name))

        return {'return': 0, 'end_html': end_html}

    return page_list(st, params)


def page_list(st, params):
    import pandas as pd
    import numpy as np

    # Read all contributors
    r = cmind.access({'action': 'find',
                      'automation': 'contributor,68eae17b590d4f8f'})
    if r['return'] > 0:
        return r

    lst = r['list']

    # Prepare the latest contributors
    all_data = []
    keys = [
        ('name', 'Name', 400, 'leftAligned'),
        ('points', 'Points', 80, 'rightAligned'),
        #            ('ongoing_number', 'Ongoing challenges', 80, 'rightAligned'),
        ('trophies', 'Trophies', 80, 'rightAligned')
    ]

    url_prefix = st.config.get_option('server.baseUrlPath') + '/'

    md_people = ''
    md_org = ''
#    for l in sorted(lst, key=lambda x: (-int(x.meta.get('last_participation_date','0')),
# for l in sorted(lst, key=lambda x: x.meta.get('name',
# x.meta.get('organization','')).lower()):

    for l in lst:

        row = {}

        m = l.meta

        # Skip from stats
        if m.get('skip', False):
            continue

        lpd = m.get('last_participation_date', '')
        trophies = m.get('trophies', [])
        ongoing = m.get('ongoing', [])

#        if lpd=='-' or (lpd!='' and int(lpd)<2023) :
#            continue
#
#        if len(ongoing)==0 and len(trophies)==0:
#            continue

#        if lpd!='':
        if True:
            uid = m['uid']
            alias = m['alias']
            name = m.get('name', '')
            org = m.get('organization', '')

            row['name_to_print'] = name if name != '' else org

            # Registration in the CK challenges gives 1 point
            y1 = ''
            y2 = ''
            if len(ongoing) > 0:
                y1 = '*'
                y2 = ' (ongoing)*'

            row['points'] = calculate_points(m)

            row['ongoing_number'] = len(ongoing)
            x = ''
            for t in ongoing:
                if t != '':
                    url = url_prefix + '?action=challenges&tags={}'.format(t)
                    x += '<a href="{}" target="_blank">{}</a><br>'.format(
                        url, t.replace('-', '&nbsp;').replace(',', '&nbsp;'))

            row['ongoing'] = x

            name2 = ''

            if name != '':
                url = misc.make_url(name, alias=uid, md=False)
                md_people += '* ' + misc.make_url(name, alias=uid) + '\n'

                if org != '':
                    name2 = ' ({})'.format(org)

            elif org != '':
                url = misc.make_url(org, alias=alias, md=False)
                md_org += '* ' + misc.make_url(org, alias=alias) + '\n'
                name = org

            row['name'] = '<a href="{}" target="_blank">{}</a><i>{}</i>'.format(
                url_prefix + url, name, name2)

            row['trophies_number'] = len(trophies)
            x = ''
            for t in trophies:
                url = t.get('url', '')
                if url != '':
                    x += '<a href="{}" target="_blank">&#127942;</a>&nbsp;'.format(
                        url)

            row['trophies'] = x

            all_data.append(row)

    # Visualize table
    pd_keys = [v[0] for v in keys]
    pd_key_names = [v[1] for v in keys]
    pd_all_data = []
    for row in sorted(all_data, key=lambda row: (row.get('ongoing_number', 0) <= 0,
                                                 -row.get('points', 0),
                                                 -row.get('trophies_number', 0),
                                                 name_to_sort(row))):
        pd_row = []
        for k in pd_keys:
            pd_row.append(row.get(k))
        pd_all_data.append(pd_row)

    df = pd.DataFrame(pd_all_data, columns=pd_key_names)

    df.index += 1

    x = '''
        <center>
         <i>
          <i>
           Check <a href="{}?action=challenges">on-going challenges</a>
           and register <a href="https://github.com/mlcommons/ck/blob/master/platform/register.md">here</a>
           to be added to this leaderboard.
          </i>
         </i>
        </center>
        <br>
        '''.format(url_prefix)

    st.write(x, unsafe_allow_html=True)

    st.write(
        '<center>' +
        df.to_html(
            escape=False,
            justify='left') +
        '</center>',
        unsafe_allow_html=True)


#    from st_aggrid import AgGrid, GridOptionsBuilder, GridUpdateMode
#    from st_aggrid.shared import JsCode
#
#    gb = GridOptionsBuilder.from_dataframe(df, editable=False)
#
#    for k in keys:
#        gb.configure_column(
#            k[1],
#            headerName=k[1],
#            width=k[2],
#            type=k[3],
#            cellRenderer=JsCode("""
#                class UrlCellRenderer {
#                  init(params) {
#                    this.eGui = document.createElement('a');
#                    this.eGui.innerHTML = params.value;
#                  }
#                  getGui() {
#                    return this.eGui;
#                  }
#                }
#            """)
#        )
#
#    AgGrid(df,
#           gridOptions=gb.build(),
#           updateMode=GridUpdateMode.VALUE_CHANGED,
#           enable_enterprise_modules=False,
#           allow_unsafe_jscode=True)

#    st.write(grid) #, unsafe_allow_html = True)

#    st.dataframe(df)
#    st.write(df.to_html(escape = False), unsafe_allow_html = True)


#    if md_people!='':
#        st.markdown("### The latest contributors (individuals)")
#        st.markdown('Huge thanks to all our contributors for supporing this community project:')
#        st.markdown(md_people)


#    if md_org!='':
#        st.markdown("### The latest contributors (organizations)")
#        st.markdown(md_org)

    # Prepare list of all contributors

#    md = ''
#    for l in sorted(lst, key=lambda x: x.meta.get('name',x.meta.get('organization','')).lower()):
#        md += prepare_name(l.meta)
#
#    if md!='':
#       st.markdown("### All contributors (individuals and orgs)")
#       st.markdown(md)

    return {'return': 0}


def name_to_sort(meta):
    name = meta.get('name_to_print', '')

    xname = name.split(' ')

    sname = xname[-1].lower()

    return sname


def calculate_points(meta):

    points = 1

    xpoints = meta.get('points', [])
    for x in xpoints:
        points += int(x.get('point', 0))

    # Automatic challenges
    points += len(meta.get('challenges', []))
    points += len(meta.get('ongoing', []))

    return points


def prepare_name(meta):
    alias = meta['alias']
    name = meta.get('name', '')
    org = meta.get('organization', '')

    md = ''
    if name != '':
        md = '* ' + misc.make_url(name, alias=alias) + '\n'
    elif org != '':
        md = '* *' + misc.make_url(org, alias=alias) + '*\n'

    return md



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):
    os_info = i['os_info']
    env = i['env']

    q = '"' if os_info['platform'] == 'windows' else "'"

    if env.get('CM_RUN_CMD', '') == '':
        if env.get('CM_BIN_NAME', '') == '':
            x = 'run.exe' if os_info['platform'] == 'windows' else 'run.out'
            env['CM_BIN_NAME'] = x

        if os_info['platform'] == 'windows':
            env['CM_RUN_CMD'] = env.get(
                'CM_RUN_PREFIX', '') + env['CM_BIN_NAME']
            if env.get('CM_RUN_SUFFIX', '') != '':
                env['CM_RUN_CMD'] += ' ' + env['CM_RUN_SUFFIX']

        else:
            if env['CM_ENABLE_NUMACTL'].lower() in ["on", "1", "true", "yes"]:
                env['CM_ENABLE_NUMACTL'] = "1"
                CM_RUN_PREFIX = "numactl " + env['CM_NUMACTL_MEMBIND'] + ' '
            else:
                CM_RUN_PREFIX = ''

            CM_RUN_PREFIX += env.get('CM_RUN_PREFIX', '')

            env['CM_RUN_PREFIX'] = CM_RUN_PREFIX

            CM_RUN_SUFFIX = (
                env['CM_REDIRECT_OUT'] +
                ' ') if 'CM_REDIRECT_OUT' in env else ''
            CM_RUN_SUFFIX += (env['CM_REDIRECT_ERR'] +
                              ' ') if 'CM_REDIRECT_ERR' in env else ''

            env['CM_RUN_SUFFIX'] = env['CM_RUN_SUFFIX'] + \
                CM_RUN_SUFFIX if 'CM_RUN_SUFFIX' in env else CM_RUN_SUFFIX

            if env.get('CM_RUN_DIR', '') == '':
                env['CM_RUN_DIR'] = os.getcwd()

            env['CM_RUN_CMD'] = CM_RUN_PREFIX + ' ' + os.path.join(
                env['CM_RUN_DIR'], env['CM_BIN_NAME']) + ' ' + env['CM_RUN_SUFFIX']

    x = env.get('CM_RUN_PREFIX0', '')
    if x != '':
        env['CM_RUN_CMD'] = x + ' ' + env.get('CM_RUN_CMD', '')

    if os_info['platform'] != 'windows' and str(
            env.get('CM_SAVE_CONSOLE_LOG', True)).lower() not in ["no", "false", "0"]:
        logs_dir = env.get('CM_LOGS_DIR', env['CM_RUN_DIR'])
        env['CM_RUN_CMD'] += r" 2>&1 | tee " + q + os.path.join(
            logs_dir, "console.out") + q + r"; echo \${PIPESTATUS[0]} > exitstatus"

    # additional arguments and tags for measuring system informations(only if
    # 'CM_PROFILE_NVIDIA_POWER' is 'on')
    if env.get('CM_PROFILE_NVIDIA_POWER', '') == "on":
        env['CM_SYS_UTILISATION_SCRIPT_TAGS'] = ''
        # this section is for selecting the variation
        if env.get('CM_MLPERF_DEVICE', '') == "gpu":
            env['CM_SYS_UTILISATION_SCRIPT_TAGS'] += ',_cuda'
        elif env.get('CM_MLPERF_DEVICE', '') == "cpu":
            env['CM_SYS_UTILISATION_SCRIPT_TAGS'] += ',_cpu'
        # this section is for supplying the input arguments/tags
        env['CM_SYS_UTILISATION_SCRIPT_TAGS'] += ' --log_dir=\'' + \
            logs_dir + '\''   # specify the logs directory
        # specifying the interval in which the system information should be
        # measured
        if env.get('CM_SYSTEM_INFO_MEASUREMENT_INTERVAL', '') != '':
            env['CM_SYS_UTILISATION_SCRIPT_TAGS'] += ' --interval=\"' + \
                env['CM_SYSTEM_INFO_MEASUREMENT_INTERVAL'] + '\"'

    # generate the pre run cmd - recording runtime system infos
    pre_run_cmd = ""

    if env.get('CM_PRE_RUN_CMD_EXTERNAL', '') != '':
        pre_run_cmd += env['CM_PRE_RUN_CMD_EXTERNAL']

    if env.get('CM_PROFILE_NVIDIA_POWER', '') == "on":
        if pre_run_cmd != '':
            pre_run_cmd += ' && '

        # running the script as a process in background
        pre_run_cmd = pre_run_cmd + 'cm run script --tags=runtime,system,utilisation' + \
            env['CM_SYS_UTILISATION_SCRIPT_TAGS'] + ' --quiet  & '
        # obtain the command if of the background process
        pre_run_cmd += r" cmd_pid=\$!  && echo CMD_PID=\$cmd_pid"
        print(
            f"Pre run command for recording the runtime system information: {pre_run_cmd}")

    env['CM_PRE_RUN_CMD'] = pre_run_cmd

    # generate the post run cmd - for killing the process that records runtime
    # system infos
    post_run_cmd = ""
    if env.get('CM_PROFILE_NVIDIA_POWER', '') == "on":
        post_run_cmd += r"echo killing process \$cmd_pid && kill -TERM \${cmd_pid}"
        print(
            f"Post run command for killing the process that measures the runtime system information: {post_run_cmd}")

    env['CM_POST_RUN_CMD'] = post_run_cmd

    # Print info
    print('***************************************************************************')
    print('CM script::benchmark-program/run.sh')
    print('')
    print('Run Directory: {}'.format(env.get('CM_RUN_DIR', '')))

    print('')
    print('CMD: {}'.format(env.get('CM_RUN_CMD', '')))

    print('')

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



import os
import tempfile
from typing import Dict, Tuple, Optional, List, Any, Union

if os.environ.get("CM_TVM_FRONTEND_FRAMEWORK", None) == "pytorch":
    import torch
    import torchvision

import tvm
from tvm import relay, meta_schedule
from tvm.driver.tvmc.frontends import load_model


def get_shape_dict_from_onnx(
    shape: List[int],
    model_path: str
) -> Dict[str, List[int]]:
    import onnx
    onnx_model = onnx.load(model_path)
    if len(shape) == 1:
        for _input in onnx_model.graph.input:
            tensor_type = _input.type.tensor_type
            if (tensor_type.HasField("shape")):
                for dimension in tensor_type.shape.dim:
                    if dimension.dim_value != 0:
                        shape.append(dimension.dim_value)
    input_all = [node.name for node in onnx_model.graph.input]
    input_initializer = [node.name for node in onnx_model.graph.initializer]
    net_feed_input = list(set(input_all) - set(input_initializer))
    return {input_name: shape for input_name in net_feed_input}


def get_mod_params(
    model_path: str,
    model_name: str,
    batch_size: int,
    frontend: str,
    input_shapes_str: Optional[str] = None,
    input_layer_name: Optional[str] = None,
    num_channels: Optional[int] = None,
    image_width: Optional[int] = None,
    image_height: Optional[int] = None,
    max_seq_length: Optional[int] = None
) -> Tuple[tvm.IRModule, Dict[str, tvm.nd.NDArray]]:
    if not input_shapes_str and (
            not image_width or not image_height) and not max_seq_length and frontend != "onnx":
        raise RuntimeError(
            "Error: None of environment variables storing shape is set!"
        )
    if input_shapes_str:
        shape_dict = eval(
            '{' +
            input_shapes_str.replace(
                'BATCH_SIZE',
                str(batch_size)) +
            '}')
    else:
        shape = []
        if image_width and image_height:
            shape = [batch_size, num_channels, image_height, image_width]
        elif max_seq_length:
            shape = [batch_size, max_seq_length]
        if frontend == "onnx":
            shape_dict = get_shape_dict_from_onnx(
                shape if len(shape) > 0 else [batch_size], model_path)
        else:
            raise RuntimeError(
                "Error: Cannot find proper shapes in environment variables"
            )
    print(f"Shape dict {shape_dict}")
    if frontend == "pytorch":
        torch_model = getattr(torchvision.models, model_name)(weights=None)
        torch_model.load_state_dict(torch.load(model_path))
        torch_model.fc = torch.nn.Sequential(
            torch_model.fc,
            torch.nn.Softmax(dim=1)
        )
        torch_model = torch_model.eval()
        shape_list = list(shape_dict.items())
        input_data = torch.randn(shape_list[0][1])
        traced_model = torch.jit.trace(torch_model, input_data).eval()
        mod, params = tvm.relay.frontend.from_pytorch(traced_model, shape_list)
    else:
        tvmc_model = load_model(path=model_path, shape_dict=shape_dict)
        mod, params = tvm.relay.transform.DynamicToStatic()(
            tvmc_model.mod), tvmc_model.params

    input_layer_name_file = os.path.join(os.getcwd(), "input_layer_name")
    if not input_layer_name:
        input_layer_name = shape_dict.keys()[0]
    with open(input_layer_name_file, 'w') as file:
        file.write(input_layer_name)

    return mod, params


def tune_model(
    mod: tvm.IRModule,
    params: Dict[str, tvm.nd.NDArray],
    target: tvm.target.Target,
) -> Tuple[str, meta_schedule.database.Database]:
    work_dir = os.path.join(os.getcwd(), "metaschedule_workdir")
    if not os.path.exists(work_dir):
        os.mkdir(work_dir)
    print("Extracting tasks...")
    extracted_tasks = meta_schedule.relay_integration.extract_tasks(
        mod, target, params
    )
    tasks, task_weights = meta_schedule.relay_integration.extracted_tasks_to_tune_contexts(
        extracted_tasks, work_dir, strategy="evolutionary"
    )

    print("Begin tuning...")
    evaluator_config = meta_schedule.runner.config.EvaluatorConfig(
        number=1,
        repeat=10,
        enable_cpu_cache_flush=True
    )
    database = meta_schedule.tune.tune_tasks(
        tasks=tasks,
        task_weights=task_weights,
        work_dir=work_dir,
        max_trials_global=10000,
        num_trials_per_iter=64,
        max_trials_per_task=512,
        builder=meta_schedule.builder.LocalBuilder(),
        runner=meta_schedule.runner.LocalRunner(
            evaluator_config=evaluator_config
        ),
    )

    return work_dir, database


def compile_model(
    mod: tvm.IRModule,
    params: Dict[str, tvm.nd.NDArray],
    work_dir: str,
    target: tvm.target.Target,
    opt_level: int,
    build_conf: Dict[str, Any],
    use_vm: bool,
    database: Optional[meta_schedule.database.Database] = None,
) -> Union[tvm.runtime.Module, tvm.runtime.vm.Executable]:
    if work_dir != '':
        if not database:
            database = meta_schedule.database.JSONDatabase(
                f"{work_dir}/database_workload.json",
                f"{work_dir}/database_tuning_record.json",
                allow_missing=False
            )
        build_conf["relay.backend.use_meta_schedule"] = True
        with tvm.transform.PassContext(
            opt_level=opt_level,
            config=build_conf
        ):
            lib = meta_schedule.relay_integration.compile_relay(
                database=database,
                mod=mod,
                target=target,
                params=params,
                backend="vm" if use_vm else "graph"
            )
    else:
        with tvm.transform.PassContext(
            opt_level=opt_level,
            config=build_conf,
        ):
            if use_vm:
                lib = tvm.relay.backend.vm.compile(
                    mod=mod,
                    target=target,
                    params=params
                )
            else:
                lib = tvm.relay.build(
                    mod,
                    target=target,
                    params=params
                )
    return lib


def serialize_vm(
    vm_exec: tvm.runtime.vm.Executable
) -> tvm.runtime.Module:
    path_consts = os.path.join(
        tempfile.mkdtemp(
            dir=os.getcwd(),
            suffix="-tvm-tmp"
        ),
        "consts"
    )
    code_path = os.path.join(os.getcwd(), "vm_exec_code.ro")
    vm_exec.move_late_bound_consts(path_consts, byte_limit=256)
    code, lib = vm_exec.save()
    with open(code_path, "wb") as file:
        file.write(code)
    return lib


def main() -> None:
    model_path = os.environ.get('CM_ML_MODEL_FILE_WITH_PATH', None)
    compiled_model = os.path.join(os.getcwd(), 'model-tvm.so')
    print('TVM model: ' + model_path)
    if model_path.endswith('.so') or model_path.endswith('.dylib'):
        compiled_model = model_path
        if not os.path.isfile(compiled_model):
            print('')
            raise RuntimeError(
                f"Error: Model file {compiled_model} not found!"
            )
    else:
        mod, params = get_mod_params(
            model_path=os.environ.get('CM_ML_MODEL_FILE_WITH_PATH', None),
            model_name=os.environ.get('CM_ML_MODEL', '').strip().lower(),
            batch_size=int(os.environ.get('CM_ML_MODEL_MAX_BATCH_SIZE', 1)),
            frontend=os.environ.get("CM_TVM_FRONTEND_FRAMEWORK", None),
            input_shapes_str=os.environ.get('CM_ML_MODEL_INPUT_SHAPES', None),
            input_layer_name=os.environ.get(
                'CM_ML_MODEL_INPUT_LAYER_NAME', None),
            num_channels=int(
                os.environ.get(
                    'CM_ML_MODEL_IMAGE_NUM_CHANNELS',
                    3)),
            image_width=int(os.environ.get('CM_ML_MODEL_IMAGE_WIDTH', 0)),
            image_height=int(os.environ.get('CM_ML_MODEL_IMAGE_HEIGHT', 0)),
            max_seq_length=int(
                os.environ.get(
                    'CM_ML_MODEL_MAX_SEQ_LENGTH', 0)),
        )
        opt_level = int(os.environ.get('CM_MLPERF_TVM_OPT_LEVEL', 3))
        target = os.environ.get(
            'CM_MLPERF_TVM_TARGET',
            f"llvm -num-cores {os.environ.get('CM_HOST_CPU_TOTAL_CORES', '1')}"
        )
        build_conf = {}
        target_host = None
        tvm_target = tvm.target.Target(target, host=target_host)
        tune_model_flag = os.environ.get('CM_TUNE_TVM_MODEL', 'no') == 'yes'
        work_dir = ''
        database = None
        use_vm = os.environ.get('CM_TVM_USE_VM', 'no') == 'yes'
        if tune_model_flag:
            work_dir, database = tune_model(
                mod=mod,
                params=params,
                target=tvm_target,
            )
        lib = compile_model(
            mod=mod,
            params=params,
            work_dir=work_dir if work_dir != '' else os.environ.get(
                'CM_TUNE_TVM_MODEL_WORKDIR', ''),
            target=tvm_target,
            opt_level=opt_level,
            build_conf=build_conf,
            use_vm=use_vm,
            database=database
        )
        if use_vm:
            lib = serialize_vm(
                vm_exec=lib
            )

        with open(os.path.join(os.getcwd(), "tvm_executor"), "w") as file:
            file.write("virtual_machine" if use_vm else "graph_executor")
        lib.export_library(compiled_model)
        print('TVM compiled model: ' + compiled_model)


if __name__ == "__main__":
    main()



from hip import hip

# Defining the value for hipDeviceGetAttribute
STRINGLENGTH = 256
hipDeviceAttributeClockRate = 5
hipDeviceAttributeMaxBlockDimX = 26
hipDeviceAttributeMaxBlockDimY = 27
hipDeviceAttributeMaxBlockDimZ = 28
hipDeviceAttributeMaxGridDimX = 29
hipDeviceAttributeMaxGridDimY = 30
hipDeviceAttributeMaxGridDimZ = 31
hipDeviceAttributeMaxThreadsPerBlock = 56
hipDeviceAttributeMaxThreadsPerMultiProcessor = 57
hipDeviceAttributeMaxRegistersPerBlock = 71
hipDeviceAttributeMaxSharedMemoryPerBlock = 74
hipDeviceAttributeWarpSize = 87


def get_gpu_info():
    num_gpus = hip.hipGetDeviceCount()[1]
    all_gpu_info = []

    for i in range(num_gpus):
        gpu_info = {
            "GPU Device ID": hip.hipDeviceGetPCIBusId(STRINGLENGTH, i)[1],
            "GPU Name": i,
            "GPU compute capability": f"{hip.hipDeviceComputeCapability(i)[1]}.{hip.hipDeviceComputeCapability(i)[2]}",
            "ROCM driver version": f"{hip.hipDriverGetVersion()[1]}",
            "ROCM runtime version": hip.hipRuntimeGetVersion()[1],
            "Global memory (GiB)": hip.hipDeviceTotalMem(i)[1] / 1_073_741_824,
            "Max clock rate": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeClockRate), i)[1] / 1000} MHz",
            "Total amount of shared memory per block (Bytes)": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeMaxSharedMemoryPerBlock), i)[1]}",
            "Total number of registers available per block (Bytes)": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeMaxRegistersPerBlock), i)[1]}",
            "Warp size": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeWarpSize), i)[1]}",
            "Maximum number of threads per multiprocessor": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeMaxThreadsPerMultiProcessor), i)[1]}",
            "Maximum number of threads per block": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeMaxThreadsPerBlock), i)[1]}",
            "Max dimension size of a thread block X": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeMaxBlockDimX), i)[1]}",
            "Max dimension size of a thread block Y": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeMaxBlockDimY), i)[1]}",
            "Max dimension size of a thread block Z": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeMaxBlockDimZ), i)[1]}",
            "Max dimension size of a grid size X": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeMaxGridDimX), i)[1]}",
            "Max dimension size of a grid size Y": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeMaxGridDimY), i)[1]}",
            "Max dimension size of a grid size Z": f"{hip.hipDeviceGetAttribute(hip.hipDeviceAttribute_t(hipDeviceAttributeMaxGridDimZ), i)[1]}",
        }
        all_gpu_info.append(gpu_info)

    return all_gpu_info


if __name__ == "__main__":
    gpu_info_list = get_gpu_info()
    with open("tmp-run.out", "w") as f:
        for idx, gpu_info in enumerate(gpu_info_list):
            print(f"GPU {idx}:")
            for key, value in gpu_info.items():
                f.write(f"{key}: {value}\n")



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import cmind as cm
from cmind import utils

import os
import subprocess
import csv
import json
import copy


file_summary = 'summary.csv'
file_summary_json = 'mlperf-training-summary-{}.json'
file_summary2 = 'summary.xlsx'
file_result = 'cm-result.json'

model2task = {
    "resnet": "image-classification",
    "maskrcnn": "object-detection-heavy-weight",
    "ssd": "object-detection-light-weight",
    "minigo": "reinforcement-learning",
    "rnnt": "speech-recognition",
    "bert": "language-processing",
    "dlrm": "recommendation",
    "3dunet": "image-segmentation"
}

model2dataset = {
    "resnet": "ImageNet",
    "maskrcnn": "COCO",
    "ssd": "OpenImages",
    "minigo": "Go",
    "rnnt": "LibriSpeech",
    "bert": "Wikipedia",
    "dlrm": "1TB Clickthrough",
    "3dunet": "KiTS19"
}


model2accuracy = {
    "resnet": 75.9,
    "maskrcnn": 0.377,
    "ssd": 34.0,
    "minigo": 50,
    "rnnt": 0.058,
    "bert": 0.72,
    "dlrm": 0.8025,
    "3dunet": 0.908
}

model2accuracy_metric = {
    "resnet": "% classification",
    "maskrcnn": "Box min AP",
    "ssd": "% mAP",
    "minigo": "% win rate vs. checkpoint",
    "rnnt": "Word Error Rate",
    "bert": "Mask-LM accuracy",
    "dlrm": "AUC",
    "3dunet": "Mean DICE score"
}


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}

    env = i['env']

    cur_dir = os.getcwd()

    # Clean summary files
    for f in [file_summary, file_summary2]:
        if os.path.isfile(f):
            os.remove(f)

    # Query cache for results dirs
    r = cm.access({'action': 'find',
                   'automation': 'cache,541d6f712a6b464e',
                   'tags': 'get,repo,mlperf-training-results'})
    if r['return'] > 0:
        return r

    lst = r['list']

    for c in lst:
        path = os.path.join(c.path, 'repo')

        if os.path.isdir(path):

            meta = c.meta

            tags = meta['tags']

            version = ''
            for t in tags:
                if t.startswith('version-'):
                    version = t[8:]
                    break

            # Run MLPerf logger
            run_script_input = i['run_script_input']
            automation = i['automation']

            env['CM_MLPERF_TRAINING_REPO_PATH'] = path
            env['CM_MLPERF_TRAINING_CURRENT_DIR'] = cur_dir
            env['CM_MLPERF_TRAINING_REPO_VERSION'] = version

            print('')
            print('Repo path:    {}'.format(path))
            print('Repo version: {}'.format(version))

            r = automation.run_native_script({'run_script_input': run_script_input,
                                              'env': env,
                                              'script_name': 'run_mlperf_logger'})
            if r['return'] > 0:
                return r

            r = convert_summary_csv_to_experiment(path, version, env)
            if r['return'] > 0:
                return r

    return {'return': 0}


def convert_summary_csv_to_experiment(path, version, env):
    print('* Processing MLPerf training results repo in cache path: {}'.format(path))

    cur_dir = os.getcwd()

    # Get Git URL
    os.chdir(path)

    burl = subprocess.check_output(
        ['git', 'config', '--get', 'remote.origin.url'])
    url = burl.decode('UTF-8').strip()

    print('  Git URL: {}'.format(url))

    os.chdir(cur_dir)

    if not os.path.isfile(file_summary):
        return {'return': 1,
                'error': '{} was not created'.format(file_summary)}
    else:
        summary = []

        with open(file_summary, encoding='utf-8') as fcsv:
            csv_reader = csv.DictReader(fcsv)

            for rows in csv_reader:
                result = {}

                keys = rows.keys()

                for k in keys:
                    v = rows[k]

                    if v == 'False':
                        v = False
                    elif v == 'True':
                        v = True
                    else:
                        try:
                            v = float(v)

                            if v == int(v):
                                v = int(v)
                        except ValueError:
                            pass

                    result[k] = v

                # Add extra tags
                if url != '':
                    result['git_url'] = url

                    location = result.get('Location', '')
                    if location != '':
                        result['url'] = url + '/tree/master/' + location

                if result.get('Accuracy', 0) > 0:
                    result['Accuracy_div_100'] = float(
                        '{:.5f}'.format(result['Accuracy'] / 100))

                # Add ratios

                # Append to summary
                summary.append(result)

        r = utils.save_json(file_summary_json.format(version), summary)
        if r['return'] > 0:
            return r

        # Create virtual experiment entries
        experiment = {}

        for result in summary:

            for model in model2task:
                if result.get(model, '') != '':
                    result1 = {}

                    result1['Result'] = result[model]
                    result1['Result_Units'] = 'min.'
                    result1['Accuracy'] = model2accuracy[model]
                    result1['Accuracy_Metric'] = model2accuracy_metric[model]
                    result1['Task'] = model2task[model]
                    result1['Benchmark'] = model2task[model]
                    result1['Dataset'] = model2dataset[model]
                    result1['Model_ID'] = model

                    result1['_Result'] = result[model]
                    result1['_Result_Units'] = 'min.'
                    result1['_Accuracy'] = model2accuracy[model]
                    result1['_Accuracy_Metric'] = model2accuracy_metric[model]
                    result1['_Task'] = model2task[model]
                    result1['_Dataset'] = model2dataset[model]
                    result1['_Model_ID'] = model

                    result1['version'] = version
                    result1['_version'] = version
                    result1['Organization'] = result['submitter']
                    result1['_Organization'] = result['submitter']
                    result1['_System'] = result['system']

                    for k in result:
                        if k == model or k not in model2task:
                            result1[k] = result[k]

                    xdivision = result['division']

                    name = 'mlperf-training--{}--' + \
                        xdivision + '--' + model2task[model]

                    name_all = name.format('all')
                    name_ver = name.format(version)

                    for name in [name_all, name_ver]:
                        if name not in experiment:
                            experiment[name] = []
                        experiment[name].append(result1)

        # Checking experiment
        env_target_repo = env.get(
            'CM_IMPORT_MLPERF_TRAINING_TARGET_REPO', '').strip()
        target_repo = '' if env_target_repo == '' else env_target_repo + ':'

        print('')
        for name in experiment:
            print('    Preparing experiment artifact "{}"'.format(name))

            tags = name.split('--')
            if 'mlperf' not in tags:
                tags.insert(0, 'mlperf')

            # Checking if experiment already exists
            r = cm.access({'action': 'find',
                           'automation': 'experiment,a0a2d123ef064bcb',
                           'artifact': target_repo + name})
            if r['return'] > 0:
                return r

            lst = r['list']

            if len(lst) == 0:
                r = cm.access({'action': 'add',
                               'automation': 'experiment,a0a2d123ef064bcb',
                               'artifact': target_repo + name,
                               'tags': tags})
                if r['return'] > 0:
                    return r

                path = r['path']
            else:
                path = lst[0].path

            results = experiment[name]

            # Check if already date directory
            dirs = os.listdir(path)

            path2 = ''
            for d in dirs:
                dd = os.path.join(path, d)
                if os.path.isdir(dd):
                    path2 = dd
                    break

            if path2 == '':

                r = utils.get_current_date_time({})
                if r['return'] > 0:
                    return r

                date_time = r['iso_datetime'].replace(
                    ':', '-').replace('T', '.')

                path2 = os.path.join(path, date_time)

                os.makedirs(path2)

            # Check if cm-result.json
            fresult = os.path.join(path2, file_result)

            if os.path.isfile(fresult):
                r = utils.load_json(fresult)
                if r['return'] > 0:
                    return r

                existing_results = r['meta']

                # Need to check which ones to add
                for result in existing_results:
                    found = False

                    # New results
                    for result2 in results:
                        matched = True

                        # Need to iterate over keys in the new results since
                        # old results can have more keys (derivates, etc)
                        for k in result2:
                            if k != 'uid':
                                if k not in result or result2[k] != result[k]:
                                    matched = False
                                    break

                        if matched:
                            found = True
                            break

                    if not found:
                        results.append(result)

            # Check extra keys
            final_results = []
            for result in results:
                # Generate UID
                if 'uid' not in result:
                    r = utils.gen_uid()
                    if r['return'] > 0:
                        return r

                    result['uid'] = r['uid']

            # Write results
            r = utils.save_json(fresult, results)
            if r['return'] > 0:
                return r

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    state = i['state']
    meta = i['meta']
    script_path = i['run_script_input']['path']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    models = env['MODELS'].split(",")

    input_backends = env.get('BACKENDS')
    if input_backends:
        input_backends = input_backends.split(",")

    devices = env.get('DEVICES')
    if devices:
        devices = devices.split(",")

    implementation = env['IMPLEMENTATION']

    power = env.get('POWER', '')

    if str(power).lower() in ["yes", "true"]:
        POWER_STRING = " --power=yes --adr.mlperf-power-client.power_server=" + env.get(
            'POWER_SERVER',
            '192.168.0.15') + " --adr.mlperf-power-client.port=" + str(
            env.get(
                'POWER_SERVER_PORT',
                '4950')) + " "
    else:
        POWER_STRING = ""

    if not devices:
        return {
            'return': 1, 'error': 'No device specified. Please set one or more (comma separated) of {cpu, qaic, cuda, rocm} for --env.DEVICES=<>'}

    cmds = []
    run_script_content = '#!/bin/bash\n\n'
    run_script_content += "POWER_STRING=\"" + POWER_STRING + "\"\n"
    run_script_content += "DIVISION=\"" + env['DIVISION'] + "\"\n"
    run_script_content += "CATEGORY=\"" + env['CATEGORY'] + "\"\n"
    run_script_content += "EXTRA_ARGS=\"" + env.get('EXTRA_ARGS', '') + "\"\n"
    run_script_content += 'source ' + \
        os.path.join(script_path, "run-template.sh") + \
        "\nPOWER_STRING=\"" + POWER_STRING + "\"\n\n"

    run_file_name = 'tmp-' + implementation + '-run'

    for model in models:
        env['MODEL'] = model

        if "mobilenets" in model:
            cmd = 'export extra_option=""'
            cmds.append(cmd)
            cmd = 'export extra_tags=""'
            cmds.append(cmd)
            assemble_tflite_cmds(cmds)
            cmd = 'export extra_option=" --adr.mlperf-inference-implementation.compressed_dataset=on"'
            cmds.append(cmd)
            assemble_tflite_cmds(cmds)

            if env.get('CM_HOST_CPU_ARCHITECTURE', '') == "aarch64":
                extra_tags = ",_armnn,_use-neon"
                cmd = f'export extra_tags="{extra_tags}"'
                cmds.append(cmd)
                assemble_tflite_cmds(cmds)
                cmd = 'export extra_option=" --adr.mlperf-inference-implementation.compressed_dataset=on"'
                cmds.append(cmd)
                assemble_tflite_cmds(cmds)

            continue

        if not input_backends:
            backends = None
            if implementation == "reference":
                if model == "resnet50":
                    backends = "tf,onnxruntime"
                elif model == "retinanet":
                    backends = "onnxruntime,pytorch"
                elif "bert" in model:
                    backends = "tf,onnxruntime,pytorch"
                elif "3d-unet" in model:
                    backends = "tf,onnxruntime,pytorch"
                elif model == "rnnt":
                    backends = "pytorch"
                elif "gptj" in model:
                    backends = "pytorch"
                elif "stable-diffusion-xl" in model:
                    backends = "pytorch"
                elif "llama2-70b" in model:
                    backends = "pytorch"
            if not backends:
                return {
                    'return': 1, 'error': f'No backend specified for the model: {model}.'}
            backends = backends.split(",")

        else:
            backends = input_backends

        for backend in backends:

            for device in devices:
                add_to_run_cmd = ''
                offline_target_qps = (
                    ((state.get(
                        model,
                        {})).get(
                        device,
                        {})).get(
                        backend,
                        {})).get('offline_target_qps')
                if offline_target_qps:
                    add_to_run_cmd += f" --offline_target_qps={offline_target_qps}"
                server_target_qps = (
                    ((state.get(
                        model,
                        {})).get(
                        device,
                        {})).get(
                        backend,
                        {})).get('server_target_qps')
                if server_target_qps:
                    add_to_run_cmd += f" --server_target_qps={server_target_qps}"

                else:  # try to do a test run with reasonable number of samples to get and record the actual system performance
                    if device == "cpu":
                        if model == "resnet50":
                            test_query_count = 1000
                        else:
                            test_query_count = 100
                    else:
                        if model == "resnet50":
                            test_query_count = 40000
                        else:
                            test_query_count = 2000
                    cmd = f'run_test "{model}" "{backend}" "{test_query_count}" "{implementation}" "{device}" "$find_performance_cmd"'
                    cmds.append(cmd)
                    # second argument is unused for submission_cmd
                cmd = f'run_test "{model}" "{backend}" "100" "{implementation}" "{device}" "$submission_cmd" "{add_to_run_cmd}"'

                singlestream_target_latency = (
                    ((state.get(
                        model,
                        {})).get(
                        device,
                        {})).get(
                        backend,
                        {})).get('singlestream_target_latency')
                if singlestream_target_latency:
                    cmd += f" --singlestream_target_latency={singlestream_target_latency}"

                cmds.append(cmd)

    run_script_content += "\n\n" + "\n\n".join(cmds)

    with open(os.path.join(script_path, run_file_name + ".sh"), 'w') as f:
        f.write(run_script_content)
    print(run_script_content)

    run_script_input = i['run_script_input']
    r = automation.run_native_script(
        {'run_script_input': run_script_input, 'env': env, 'script_name': run_file_name})

    if r['return'] > 0:
        return r

    return {'return': 0}


def assemble_tflite_cmds(cmds):
    cmd = 'run "$tflite_accuracy_cmd"'
    cmds.append(cmd)
    cmd = 'run "$tflite_performance_cmd"'
    cmds.append(cmd)
    cmd = 'run "$tflite_readme_cmd"'
    cmds.append(cmd)
    return


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    env['CM_GIT_CHECKOUT'] = "v" + env['CM_VERSION']
    quiet = (env.get('CM_QUIET', False) == 'yes')

    return {'return': 0}


def postprocess(i):

    env = i['env']
    if '+C_INCLUDE_PATH' not in env:
        env['+C_INCLUDE_PATH'] = []
    if '+LD_LIBRARY_PATH' not in env:
        env['+LD_LIBRARY_PATH'] = []

    gtest_install_path = os.path.join(os.getcwd(), "install")
    env['CM_GOOGLE_TEST_SRC_PATH'] = env['CM_GIT_REPO_CHECKOUT_PATH']
    env['CM_GOOGLE_TEST_INSTALL_PATH'] = gtest_install_path
    env['+C_INCLUDE_PATH'].append(os.path.join(gtest_install_path, "include"))
    env['+LD_LIBRARY_PATH'].append(os.path.join(gtest_install_path, "lib"))

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    automation = i['automation']

    cm = automation.cmind

    path = os.getcwd()

    model_stub = env.get('CM_MODEL_ZOO_STUB', '')
    if model_stub == '':

        variations = list(i.get('meta', {}).get('variations', {}).keys())

        variation_models = []
        for v in variations:
            if '#' not in v:
                variation_models.append(v)

        return {'return': 1, 'error': 'ENV CM_MODEL_ZOO_STUB is not set. Please select variation from {}'.format(
            str(variation_models))}

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']

    env = i['env']

    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_ML_MODEL_FILE_WITH_PATH']

    onnx_path = os.path.join(env['CM_ML_MODEL_FILE_WITH_PATH'], "model.onnx")

    if os.path.exists(onnx_path):
        env['CM_MLPERF_CUSTOM_MODEL_PATH'] = onnx_path

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os


def preprocess(i):

    os_info = i['os_info']

    xsep = ';' if os_info['platform'] == 'windows' else ':'

    env = i['env']
    results_dir = env.get("CM_MLPERF_ACCURACY_RESULTS_DIR", "")

    if results_dir == "":
        print("Please set CM_MLPERF_ACCURACY_RESULTS_DIR")
        return {'return': -1}

    # In fact, we expect only 1 command line here
    run_cmds = []

    if env.get('CM_MAX_EXAMPLES', '') != '' and env.get(
            'CM_MLPERF_RUN_STYLE', '') != 'valid':
        max_examples_string = " --max_examples " + env['CM_MAX_EXAMPLES']
    else:
        max_examples_string = ""

    results_dir_split = results_dir.split(xsep)
    dataset = env['CM_DATASET']
    regenerate_accuracy_file = env.get(
        'CM_MLPERF_REGENERATE_ACCURACY_FILE', env.get(
            'CM_RERUN', False))

    for result_dir in results_dir_split:

        out_file = os.path.join(result_dir, 'accuracy.txt')

        if os.path.exists(out_file) and (
                os.stat(out_file).st_size != 0) and not regenerate_accuracy_file:
            continue

        if dataset == "openimages":
            if env.get('CM_DATASET_PATH_ROOT', '') != '':
                dataset_dir = env['CM_DATASET_PATH_ROOT']
                if 'DATASET_ANNOTATIONS_FILE_PATH' in env:
                    del (env['DATASET_ANNOTATIONS_FILE_PATH'])
            else:
                env['DATASET_ANNOTATIONS_FILE_PATH'] = env['CM_DATASET_ANNOTATIONS_FILE_PATH']
                dataset_dir = os.getcwd()  # not used, just to keep the script happy
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " " + "'" + os.path.join(env['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH'], "tools",
                                                                            "accuracy-openimages.py") + "'" + " --mlperf-accuracy-file " + "'" + os.path.join(result_dir,
                                                                                                                                                              "mlperf_log_accuracy.json") + "'" + " --openimages-dir " + "'" + dataset_dir + "'" + " --verbose > " + "'" + \
                out_file + "'"

        elif dataset == "imagenet":
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + os.path.join(env['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH'], "tools",
                                                                       "accuracy-imagenet.py") + "' --mlperf-accuracy-file '" + os.path.join(result_dir,
                                                                                                                                             "mlperf_log_accuracy.json") + "' --imagenet-val-file '" + os.path.join(env['CM_DATASET_AUX_PATH'],
                                                                                                                                                                                                                    "val.txt") + "' --dtype " + env.get('CM_ACCURACY_DTYPE', "float32") + " > '" + out_file + "'"

        elif dataset == "squad":
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + os.path.join(env['CM_MLPERF_INFERENCE_BERT_PATH'],
                                                                       "accuracy-squad.py") + "' --val_data '" + env['CM_DATASET_SQUAD_VAL_PATH'] + \
                "' --log_file '" + os.path.join(result_dir, "mlperf_log_accuracy.json") + \
                "' --vocab_file '" + env['CM_ML_MODEL_BERT_VOCAB_FILE_WITH_PATH'] + \
                "' --out_file '" + os.path.join(result_dir, 'predictions.json') + \
                "' --features_cache_file '" + os.path.join(env['CM_MLPERF_INFERENCE_BERT_PATH'], 'eval_features.pickle') + \
                "' --output_dtype " + env['CM_ACCURACY_DTYPE'] + env.get(
                'CM_OUTPUT_TRANSPOSED', '') + max_examples_string + " > '" + out_file + "'"

        elif dataset == "cnndm":
            if env.get('CM_MLPERF_IMPLEMENTATION', '') == 'intel':
                accuracy_checker_file = env['CM_MLPERF_INFERENCE_INTEL_GPTJ_ACCURACY_FILE_WITH_PATH']
                env['+PYTHONPATH'] = [os.path.dirname(env['CM_MLPERF_INFERENCE_INTEL_GPTJ_DATASET_FILE_WITH_PATH'])] + [
                    os.path.dirname(env['CM_MLPERF_INFERENCE_INTEL_GPTJ_DATASET_ITEM_FILE_WITH_PATH'])] + env['+PYTHONPATH']
                suffix_string = " --model-name-or-path '" + \
                    env['GPTJ_CHECKPOINT_PATH'] + "'"
            else:
                accuracy_checker_file = os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], "language", "gpt-j",
                                                     "evaluation.py")
                suffix_string = " --dtype " + \
                    env.get('CM_ACCURACY_DTYPE', "float32")
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + accuracy_checker_file + "' --mlperf-accuracy-file '" + os.path.join(result_dir, "mlperf_log_accuracy.json") + \
                "' --dataset-file '" + \
                env['CM_DATASET_EVAL_PATH'] + "'" + \
                suffix_string + " > '" + out_file + "'"

        elif dataset == "openorca":
            accuracy_checker_file = os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], "language", "llama2-70b",
                                                 "evaluate-accuracy.py")
            if env.get('CM_VLLM_SERVER_MODEL_NAME', '') == '':
                checkpoint_path = env['CM_ML_MODEL_LLAMA2_FILE_WITH_PATH']
            else:
                checkpoint_path = env['CM_VLLM_SERVER_MODEL_NAME']
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + accuracy_checker_file + "' --checkpoint-path '" + checkpoint_path + "' --mlperf-accuracy-file '" + os.path.join(result_dir, "mlperf_log_accuracy.json") + \
                "' --dataset-file '" + env['CM_DATASET_PREPROCESSED_PATH'] + "'" + " --dtype " + env.get(
                    'CM_ACCURACY_DTYPE', "int32") + " > '" + out_file + "'"

        elif dataset == "openorca-gsm8k-mbxp-combined":
            accuracy_checker_file = os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], "language", "mixtral-8x7b",
                                                 "evaluate-accuracy.py")
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + accuracy_checker_file + "' --checkpoint-path '" + env['MIXTRAL_CHECKPOINT_PATH'] + "' --mlperf-accuracy-file '" + os.path.join(result_dir, "mlperf_log_accuracy.json") + \
                "' --dataset-file '" + env['CM_DATASET_MIXTRAL_PREPROCESSED_PATH'] + "'" + \
                " --dtype " + env.get('CM_ACCURACY_DTYPE',
                                      "float32") + " > '" + out_file + "'"

        elif dataset == "coco2014":
            env['+PYTHONPATH'] = [
                os.path.join(
                    env['CM_MLPERF_INFERENCE_SOURCE'],
                    "text_to_image",
                    "tools"),
                os.path.join(
                    env['CM_MLPERF_INFERENCE_SOURCE'],
                    "text_to_image",
                    "tools",
                    "fid")]
            extra_options = ""

            if env.get('CM_SDXL_STATISTICS_FILE_PATH', '') != '':
                extra_options += (
                    f""" --statistics-path '{
                        env['CM_SDXL_STATISTICS_FILE_PATH']}'"""
                )

            if env.get('CM_SDXL_COMPLIANCE_IMAGES_PATH', '') != '':
                extra_options += (
                    f""" --compliance-images-path '{
                        env['CM_SDXL_COMPLIANCE_IMAGES_PATH']}' """
                )
            else:
                extra_options += f""" --compliance-images-path '{
                    os.path.join(
                        result_dir, "images")}' """

            if env.get('CM_COCO2014_SAMPLE_ID_PATH', '') != '':
                extra_options += (
                    f" --ids-path '{env['CM_COCO2014_SAMPLE_ID_PATH']}' "
                )

            if env.get('CM_SDXL_ACCURACY_RUN_DEVICE', '') != '':
                extra_options += (
                    f" --device '{env['CM_SDXL_ACCURACY_RUN_DEVICE']}' "
                )

            # env['DATASET_ANNOTATIONS_FILE_PATH'] = env['CM_DATASET_ANNOTATIONS_FILE_PATH']
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], "text_to_image", "tools",
                                                                       "accuracy_coco.py") + "' --mlperf-accuracy-file '" + os.path.join(result_dir, "mlperf_log_accuracy.json") + \
                "' --caption-path '" + os.path.join(
                env['CM_MLPERF_INFERENCE_SOURCE'],
                "text_to_image",
                "coco2014",
                "captions",
                "captions_source.tsv") + "'" + extra_options + " > '" + out_file + "'"

        elif dataset == "kits19":
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + os.path.join(env['CM_MLPERF_INFERENCE_3DUNET_PATH'],
                                                                       "accuracy_kits.py") + \
                "' --preprocessed_data_dir '" + env['CM_DATASET_PREPROCESSED_PATH'] +\
                "' --postprocessed_data_dir '" + result_dir +\
                "' --log_file '" + os.path.join(result_dir, "mlperf_log_accuracy.json") + \
                "' --output_dtype " + \
                env['CM_ACCURACY_DTYPE'] + " > '" + out_file + "'"

        elif dataset == "librispeech":
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + os.path.join(env['CM_MLPERF_INFERENCE_RNNT_PATH'],
                                                                       "accuracy_eval.py") + \
                "' --dataset_dir '" + os.path.join(env['CM_DATASET_PREPROCESSED_PATH'], "..") +\
                "' --manifest '" + env['CM_DATASET_PREPROCESSED_JSON'] +\
                "' --log_dir '" + result_dir + \
                "' --output_dtype " + \
                env['CM_ACCURACY_DTYPE'] + " > '" + out_file + "'"

        elif dataset == "terabyte":
            extra_options = ""
            if env.get('CM_DLRM_V2_AGGREGATION_TRACE_FILE_PATH', '') != '':
                extra_options += (
                    f""" --aggregation-trace-file '{
                        env['CM_DLRM_V2_AGGREGATION_TRACE_FILE_PATH']}' """
                )
            if env.get('CM_DLRM_V2_DAY23_FILE_PATH', '') != '':
                extra_options += (
                    f""" --day-23-file '{
                        env['CM_DLRM_V2_DAY23_FILE_PATH']}' """
                )
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + os.path.join(env['CM_MLPERF_INFERENCE_DLRM_V2_PATH'], "pytorch", "tools",
                                                                       "accuracy-dlrm.py") + "' --mlperf-accuracy-file '" + os.path.join(result_dir,
                                                                                                                                         "mlperf_log_accuracy.json") + "'" + extra_options + \
                " --dtype " + env.get('CM_ACCURACY_DTYPE',
                                      "float32") + " > '" + out_file + "'"

        elif dataset == "igbh":
            if env.get('CM_DATASET_IGBH_SIZE', '') == '':
                if env.get('CM_MLPERF_SUBMISSION_GENERATION_STYLE',
                           '') == "full":
                    env['CM_DATASET_IGBH_SIZE'] = "full"
                else:
                    env['CM_DATASET_IGBH_SIZE'] = "tiny"
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], "graph", "R-GAT", "tools", "accuracy_igbh.py") + "' --mlperf-accuracy-file '" + os.path.join(
                result_dir, "mlperf_log_accuracy.json") + "' --dataset-path '" + env['CM_DATASET_IGBH_PATH'] + "' --dataset-size '" + env['CM_DATASET_IGBH_SIZE'] + "' --output-file '" + out_file + "'"

        elif dataset == "dataset_llama3":
            CMD = env['CM_PYTHON_BIN_WITH_PATH'] + " '" + os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], "language", "llama3.1-405b", "evaluate-accuracy.py") + "' --checkpoint-path '" + env['CM_ML_MODEL_LLAMA3_CHECKPOINT_PATH'] + "' --mlperf-accuracy-file '" + os.path.join(
                result_dir, "mlperf_log_accuracy.json") + "' --dtype '" + env['CM_ACCURACY_DTYPE'] + "' --dataset-file '" + env['CM_DATASET_LLAMA3_PATH'] + "' > '" + out_file + "'"

        else:
            return {'return': 1, 'error': 'Unsupported dataset'}

        run_cmds.append(CMD)

    if os_info['platform'] == 'windows':
        env['CM_RUN_CMDS'] = (
            '\n'.join(run_cmds)).replace(
            "'",
            '"').replace(
            '>',
            '^>')
    else:
        env['CM_RUN_CMDS'] = "??".join(run_cmds)

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']
    env = i['env']
    state = i['state']

    xsep = ';' if os_info['platform'] == 'windows' else ':'

    results_dir = env.get("CM_MLPERF_ACCURACY_RESULTS_DIR", "")

    results_dir_split = results_dir.split(xsep)

    for result_dir in results_dir_split:
        accuracy_file = os.path.join(result_dir, "accuracy.txt")

        if os.path.exists(accuracy_file):
            print('')
            print('Accuracy file: {}'.format(accuracy_file))
            print('')

            x = ''
            with open(accuracy_file, "r") as fp:
                x = fp.read()

            if x != '':
                print(x)

                # Trying to extract accuracy dict
                for y in x.split('\n'):
                    if y.startswith('{') and y.endswith('}'):

                        import json

                        try:
                            z = json.loads(y)
                            state['app_mlperf_inference_accuracy'] = z

                            break
                        except ValueError as e:
                            pass

            print('')
    return {'return': 0}



from sparsezoo import Model
import os

model_stub = os.environ.get('CM_MODEL_ZOO_STUB', '')
print(f"Downloading model {model_stub}")
stub = f"{model_stub}"
model = Model(stub)

with open('tmp-run-env.out', 'w') as f:
    f.write(f"CM_ML_MODEL_FILE_WITH_PATH={model.path}")



# Developer: Grigori Fursin

import typing
import importlib
import os
import psutil

import utils

import numpy as np

import torch

from loadgen.model import Model, ModelFactory, ModelInput, ModelInputSampler


xinput = input


class XModel(Model):
    def __init__(self, session):
        assert session is not None
        self.session = session

    def predict(self, input: ModelInput):

        print('')
        utils.print_host_memory_use('Host memory used')

        print('Running inference ...')
        with torch.no_grad():
            output = self.session(input)

        utils.print_host_memory_use('Host memory used')

        return output


class XModelFactory(ModelFactory):
    def __init__(
        self,
        model_path: str,
        execution_provider="CPUExecutionProvider",
        execution_mode="",
        intra_op_threads=0,
        inter_op_threads=0,
        model_code='',
        model_cfg={},
        model_sample_pickle=''
    ):

        self.model_path = model_path
        self.model_code = model_code
        self.model_cfg = model_cfg
        self.model_sample_pickle = model_sample_pickle
        self.execution_provider = execution_provider

    def create(self) -> Model:
        print('')
        print('Loading model: {}'.format(self.model_path))

        if self.execution_provider == 'CPUExecutionProvider':
            torch_provider = 'cpu'
        elif self.execution_provider == 'CUDAExecutionProvider':
            torch_provider = 'cuda'
            if not torch.cuda.is_available():
                raise Exception(
                    'Error: CUDA is forced but not available or installed in PyTorch!')
        else:
            raise Exception(
                'Error: execution provider is unknown ({})!'.format(
                    self.execution_provider))

        checkpoint = torch.load(self.model_path,
                                map_location=torch.device(torch_provider))

        if self.model_code == '':
            raise Exception('Error: path to model code was not provided!')

        if self.model_sample_pickle == '':
            raise Exception(
                'Error: path to model sample pickle was not provided!')

        # Load sample
        import pickle
        with open(self.model_sample_pickle, 'rb') as handle:
            self.input_sample = pickle.load(handle)

        # Check if has CM connector
        cm_model_module = os.path.join(self.model_code, 'cmc.py')
        if not os.path.isfile(cm_model_module):
            raise Exception(
                'cm.py interface for a PyTorch model was not found in {}'.format(
                    self.model_code))

        print('')
        print('Collective Mind Connector for the model found: {}'.format(
            cm_model_module))

        # Load CM interface for the model
        import sys
        sys.path.insert(0, self.model_code)
        model_module = importlib.import_module('cmc')
        del (sys.path[0])

        # Init model
        if len(self.model_cfg) > 0:
            print('Model cfg: {}'.format(self.model_cfg))

        r = model_module.model_init(checkpoint, self.model_cfg)
        if r['return'] > 0:
            raise Exception('Error: {}'.format(r['error']))

        model = r['model']

        if torch_provider == 'cuda':
            model.cuda()

        model.eval()

        return XModel(model)


class XModelInputSampler(ModelInputSampler):
    def __init__(self, model_factory: XModelFactory):
        model = model_factory.create()
        self.input_sample = model_factory.input_sample
        return

    def sample(self, id_: int) -> ModelInput:
        input = self.input_sample
        return input



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    path = env.get('SDXL_CHECKPOINT_PATH', '').strip()

    if path == '' or not os.path.exists(path):
        env['CM_TMP_REQUIRE_DOWNLOAD'] = 'yes'

    return {'return': 0}


def postprocess(i):

    env = i['env']

    if env.get('SDXL_CHECKPOINT_PATH', '') == '':
        env['SDXL_CHECKPOINT_PATH'] = env['CM_ML_MODEL_PATH']
    elif env.get('CM_ML_MODEL_PATH', '') == '':
        env['CM_ML_MODEL_PATH'] = env['SDXL_CHECKPOINT_PATH']

    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['SDXL_CHECKPOINT_PATH']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}
    env = i['env']

    if env.get('CM_MLPERF_SKIP_RUN', '') == "yes":
        return {'return': 0}

    if 'CM_MODEL' not in env:
        return {
            'return': 1, 'error': 'Please select a variation specifying the model to run'}
    if 'CM_MLPERF_BACKEND' not in env:
        return {'return': 1,
                'error': 'Please select a variation specifying the backend'}
    if 'CM_MLPERF_DEVICE' not in env:
        return {
            'return': 1, 'error': 'Please select a variation specifying the device to run on'}

    source_files = []
    script_path = i['run_script_input']['path']

    env['CM_SOURCE_FOLDER_PATH'] = os.path.join(
        script_path, env['CM_TMP_SRC_FOLDER'])

    for file in os.listdir(env['CM_SOURCE_FOLDER_PATH']):
        if file.endswith(".c") or file.endswith(".cpp"):
            source_files.append(file)

    env['CM_CXX_SOURCE_FILES'] = ";".join(source_files)

    if '+CPLUS_INCLUDE_PATH' not in env:
        env['+CPLUS_INCLUDE_PATH'] = []

    env['+CPLUS_INCLUDE_PATH'].append(os.path.join(script_path, "inc"))
    env['+C_INCLUDE_PATH'].append(os.path.join(script_path, "inc"))

    # TODO: get cuda path ugly fix
    if env['CM_MLPERF_DEVICE'] == 'gpu':
        env['+C_INCLUDE_PATH'].append(env['CM_CUDA_PATH_INCLUDE'])
        env['+CPLUS_INCLUDE_PATH'].append(env['CM_CUDA_PATH_INCLUDE'])
        env['+LD_LIBRARY_PATH'].append(env['CM_CUDA_PATH_LIB'])
        env['+DYLD_FALLBACK_LIBRARY_PATH'].append(env['CM_CUDA_PATH_INCLUDE'])

    if '+ CXXFLAGS' not in env:
        env['+ CXXFLAGS'] = []
    env['+ CXXFLAGS'].append("-std=c++17")

    # add preprocessor flag like "#define CM_MODEL_RESNET50"
    env['+ CXXFLAGS'].append('-DCM_MODEL_' + env['CM_MODEL'].upper())
    # add preprocessor flag like "#define CM_MLPERF_BACKEND_ONNXRUNTIME"
    env['+ CXXFLAGS'].append('-DCM_MLPERF_BACKEND_' +
                             env['CM_MLPERF_BACKEND'].upper())
    # add preprocessor flag like "#define CM_MLPERF_DEVICE_CPU"
    env['+ CXXFLAGS'].append('-DCM_MLPERF_DEVICE_' +
                             env['CM_MLPERF_DEVICE'].upper())

    if '+ LDCXXFLAGS' not in env:
        env['+ LDCXXFLAGS'] = []

    env['+ LDCXXFLAGS'] += [
        "-lmlperf_loadgen",
        "-lpthread"
    ]
    # e.g. -lonnxruntime
    if 'CM_MLPERF_BACKEND_LIB_NAMESPEC' in env:
        env['+ LDCXXFLAGS'].append('-l' +
                                   env['CM_MLPERF_BACKEND_LIB_NAMESPEC'])
    # e.g. -lcudart
    if 'CM_MLPERF_DEVICE_LIB_NAMESPEC' in env:
        env['+ LDCXXFLAGS'].append('-l' + env['CM_MLPERF_DEVICE_LIB_NAMESPEC'])

    if env.get('CM_TMP_LINK_LIBS', []):
        libs = env['CM_TMP_LINK_LIBS'].split(",")
        for lib in libs:
            env['+ LDCXXFLAGS'].append(' -l' + lib)

    env['CM_LINKER_LANG'] = 'CXX'
    env['CM_RUN_DIR'] = os.getcwd()

    if 'CM_MLPERF_CONF' not in env:
        env['CM_MLPERF_CONF'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'], "mlperf.conf")
    if 'CM_MLPERF_USER_CONF' not in env:
        env['CM_MLPERF_USER_CONF'] = os.path.join(
            env['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH'], "user.conf")

    if env.get('CM_DATASET_COMPRESSED', "no").lower() in [
            "yes", "on", "true"] and "float" in env.get('CM_MLPERF_MODEL_PRECISION', ''):
        # Use all cores for input preprocessing
        env['CM_HOST_USE_ALL_CORES'] = "yes"
        env['CM_MLPERF_SUT_NAME_RUN_CONFIG_SUFFIX2'] = "with_live_preprocessing"

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import json
import shutil


def preprocess(i):
    env = i['env']
    state = i['state']
    os_info = i['os_info']

    submitter = env.get('CM_MLPERF_SUBMITTER', 'MLCommons')

    auto_detected_hw_name = False
    if env.get('CM_HW_NAME', '') == '':
        host_name = env.get('CM_HOST_SYSTEM_NAME', 'default').replace("-", "_")
        env['CM_HW_NAME'] = host_name
        auto_detected_hw_name = True

    hw_name = env['CM_HW_NAME']

    backend = env.get('CM_MLPERF_BACKEND', '')
    backend_version = env.get('CM_MLPERF_BACKEND_VERSION', '')
    sut_suffix = ''
    backend_name = ''
    backend_desc = ''
    if backend:
        backend_name = env.get('CM_MLPERF_BACKEND_NAME', backend)
        sut_suffix = "-" + backend
        backend_desc = backend_name
        if backend_version:
            sut_suffix += "-" + backend_version
            backend_desc += ' v' + backend_version

    sut = hw_name + sut_suffix
    script_path = i['run_script_input']['path']
    sut_desc_path = env['CM_MLPERF_INFERENCE_SUT_DESC_PATH']

    sut_path = os.path.join(sut_desc_path, "suts", sut + ".json")
    if os.path.exists(sut_path) and env.get('CM_SUT_DESC_CACHE', '') == "yes":
        print(f"Reusing SUT description file {sut}")
        state['CM_SUT_META'] = json.load(open(sut_path))
    else:
        if not os.path.exists(os.path.dirname(sut_path)):
            os.makedirs(os.path.dirname(sut_path))

        print("Generating SUT description file for " + sut)
        hw_path = os.path.join(os.getcwd(), "hardware", hw_name + ".json")
        if not os.path.exists(os.path.dirname(hw_path)):
            os.makedirs(os.path.dirname(hw_path))
        if not os.path.exists(hw_path):
            default_hw_path = os.path.join(
                script_path, "hardware", "default.json")
            print("HW description file for " + hw_name +
                  " not found. Copying from default!!!")
            shutil.copy(default_hw_path, hw_path)

        state['CM_HW_META'] = json.load(open(hw_path))
        state['CM_SUT_META'] = state['CM_HW_META']
        state['CM_SUT_META']['framework'] = backend_desc
        os_name = env.get('CM_HOST_OS_FLAVOR', '').capitalize()
        os_version = env.get('CM_HOST_OS_VERSION', '')
        if os_name and os_version:
            os_name_string = os_name + " " + os_version
        else:
            os_name_string = ''
        os_type = env.get('CM_HOST_OS_TYPE', '')
        kernel = env.get('CM_HOST_OS_KERNEL_VERSION', '')
        if os_type and kernel:
            os_name_string += " (" + os_type + "-" + kernel
            glibc_version = env.get('CM_HOST_OS_GLIBC_VERSION', '')
            if glibc_version:
                os_name_string += '-glibc' + glibc_version
            os_name_string += ')'
        python_version = env.get('CM_PYTHON_VERSION', '')
        compiler = env.get('CM_COMPILER_FAMILY', '')
        compiler_version = env.get('CM_COMPILER_VERSION', '')
        state['CM_SUT_META']['submitter'] = submitter

        # If Windows and os_name_string is empty, rebuild it:

        if os_name_string == '' and os_info['platform'] == 'windows':
            import platform
            os_name_string = str(platform.platform())

        state['CM_SUT_META']['operating_system'] = os_name_string

        state['CM_SUT_META']['other_software_stack'] = "Python: " + \
            python_version + ", " + compiler + "-" + compiler_version

        if env.get('CM_DOCKER_VERSION', '') != '':
            state['CM_SUT_META']['other_software_stack'] += " Docker version:" + \
                env['CM_DOCKER_VERSION']
        else:
            if os.path.exists('/.dockerenv'):
                state['CM_SUT_META']['other_software_stack'] += ", Using Docker "

        if state['CM_SUT_META'].get('system_name', '') == '':
            system_name = env.get('CM_MLPERF_SYSTEM_NAME')
            if not system_name:
                system_name = env.get('CM_HW_NAME')
                if system_name:
                    if auto_detected_hw_name:
                        system_name += " (auto detected)"
                else:
                    system_name = " (generic)"
            state['CM_SUT_META']['system_name'] = system_name

        # Add GPU info

        if env.get('CM_MLPERF_DEVICE', '') == "gpu" or env.get(
                'CM_MLPERF_DEVICE', '') == "cuda":
            if env.get('CM_CUDA_VERSION', '') != '':
                cuda_version = " , CUDA " + env['CM_CUDA_VERSION']
                state['CM_SUT_META']['other_software_stack'] += cuda_version

        if 'cm_cuda_device_prop' in state:
            state['CM_SUT_META']['accelerator_frequency'] = state['cm_cuda_device_prop']['Max clock rate']
            state['CM_SUT_META']['accelerator_memory_capacity'] = str(int(
                state['cm_cuda_device_prop']['Global memory']) / (1024 * 1024.0 * 1024)) + " GB"
            state['CM_SUT_META']['accelerator_model_name'] = state['cm_cuda_device_prop']['GPU Name']
            num_accelerators = env.get('CM_CUDA_NUM_DEVICES', "1")
            state['CM_SUT_META']['accelerators_per_node'] = num_accelerators

        if state['CM_SUT_META'].get('host_processor_core_count', '') == '':
            physical_cores_per_node = env.get(
                'CM_HOST_CPU_PHYSICAL_CORES_PER_SOCKET')

            if physical_cores_per_node is None or physical_cores_per_node == '':
                if os_info['platform'] == 'windows':
                    physical_cores_per_node = '1'

            state['CM_SUT_META']['host_processor_core_count'] = physical_cores_per_node

        if state['CM_SUT_META'].get('host_processor_model_name', '') == '':
            state['CM_SUT_META']['host_processor_model_name'] = env.get(
                'CM_HOST_CPU_MODEL_NAME', 'undefined')
        if state['CM_SUT_META'].get('host_processors_per_node', '') == '':
            x = env.get('CM_HOST_CPU_SOCKETS', '')
            if x == '' and os_info['platform'] == 'windows':
                x = '1'
            state['CM_SUT_META']['host_processors_per_node'] = x

        if state['CM_SUT_META'].get('host_processor_caches', '') == '':
            state['CM_SUT_META']['host_processor_caches'] = "L1d cache: " + env.get('CM_HOST_CPU_L1D_CACHE_SIZE', ' ') + \
                ", L1i cache: " + env.get('CM_HOST_CPU_L1I_CACHE_SIZE', ' ') + ", L2 cache: " + \
                env.get('CM_HOST_CPU_L2_CACHE_SIZE', ' ') + \
                ", L3 cache: " + env.get('CM_HOST_CPU_L3_CACHE_SIZE', ' ')

        if state['CM_SUT_META'].get('host_processor_frequency', '') == '':
            state['CM_SUT_META']['host_processor_frequency'] = env.get(
                'CM_HOST_CPU_MAX_MHZ') if env.get('CM_HOST_CPU_MAX_MHZ', '') != '' else 'undefined'
        if state['CM_SUT_META'].get('host_memory_capacity', '') == '':
            state['CM_SUT_META']['host_memory_capacity'] = env.get(
                'CM_HOST_MEMORY_CAPACITY') if env.get('CM_HOST_MEMORY_CAPACITY', '') != '' else 'undefined'
        if state['CM_SUT_META'].get('host_storage_capacity', '') == '':
            state['CM_SUT_META']['host_storage_capacity'] = env.get(
                'CM_HOST_DISK_CAPACITY') if env.get('CM_HOST_DISK_CAPACITY', '') != '' else 'undefined'
        if 'CM_SUT_SW_NOTES' in env:
            sw_notes = env['CM_SUT_SW_NOTES']
        else:
            sw_notes = ''
        state['CM_SUT_META']['sw_notes'] = sw_notes

        if env.get('CM_SUDO_USER', '') == "yes" and env.get(
                'CM_HOST_OS_TYPE', 'linux'):
            r = i['automation'].run_native_script(
                {'run_script_input': i['run_script_input'], 'env': env, 'script_name': 'detect_memory'})
            if r['return'] > 0:
                return r
            if env.get('CM_HOST_MEM_INFO', '') != '':
                state['CM_SUT_META']['host_memory_configuration'] = env['CM_HOST_MEM_INFO']

        state['CM_SUT_META'] = dict(sorted(state['CM_SUT_META'].items()))

        sut_file = open(sut_path, "w")
        json.dump(state['CM_SUT_META'], sut_file, indent=4)
        sut_file.close()

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import subprocess
import sys


def install(package):
    additional_install_options = []
    r = subprocess.run([sys.executable, "-m", "pip",
                       "--version"], check=True, capture_output=True)
    r = r.stdout.decode("utf-8")
    if "pip" in r:
        out_split = r.split(" ")
        if len(out_split) < 2:
            return {'return': 1, 'error': 'Pip version detection failed'}
        pip_version = out_split[1].split(".")
        if pip_version and len(pip_version) > 1 and int(pip_version[0]) >= 23:
            additional_install_options.append("--break-system-packages")
    run_cmd = [sys.executable, "-m", "pip", "install", package]
    run_cmd += additional_install_options
    r = subprocess.run(run_cmd, check=True)

    return {'return': 0}


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    if env.get('CM_PIP_PACKAGE_NAME', '') != '':
        r = install(env['CM_PIP_PACKAGE_NAME'])
        if r['return'] > 0:
            return r

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



# https://huggingface.co/runwayml/stable-diffusion-v1-5/tree/onnx

import os

from optimum.onnxruntime import ORTStableDiffusionPipeline

output = os.environ.get('CM_APP_STABLE_DIFFUSION_ONNX_PY_OUTPUT', '')

f = os.path.join(output, 'output.png')

if os.path.isfile(f):
    os.remove(f)

cm_model_path = os.environ.get('CM_ML_MODEL_PATH', '')
if cm_model_path == '':
    print('Error: CM_ML_MODEL_PATH env is not defined')
    exit(1)

device = os.environ.get('CM_DEVICE', '')

pipeline = ORTStableDiffusionPipeline.from_pretrained(
    cm_model_path, local_files_only=True).to(device)

text = os.environ.get('CM_APP_STABLE_DIFFUSION_ONNX_PY_TEXT', '')
if text == '':
    text = "a photo of an astronaut riding a horse on mars"


print('')
print('Generating imaged based on "{}"'.format(text))

image = pipeline(text).images[0]

image.save(f)

print('Image recorded to "{}"'.format(f))



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

#    if os_info['platform'] == 'windows':
# return {'return':1, 'error': 'Windows is not supported in this script
# yet'}

    env = i['env']
    meta = i['meta']

    if 'CM_GIT_DEPTH' not in env:
        env['CM_GIT_DEPTH'] = ''

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    env['CM_EEMBC_ENERGY_RUNNER_SRC'] = os.path.join(os.getcwd(), 'src')
    datasets_src_path = os.path.join(os.getcwd(), 'src', 'datasets')
    env['CM_EEMBC_ENERGY_RUNNER_SRC_DATASETS'] = datasets_src_path

    # Get user directory for EEMBC runner path
    home_directory = os.path.expanduser('~')

    sessions_path = os.path.join(home_directory, 'eembc', 'runner', 'sessions')

    print('')
    print('Path to EEMBC runner sessions: {}'.format(sessions_path))

    env['CM_EEMBC_ENERGY_RUNNER_SESSIONS'] = sessions_path

    if not os.path.isdir(sessions_path):
        os.makedirs(sessions_path)

    datasets_path = os.path.join(
        home_directory,
        'eembc',
        'runner',
        'benchmarks',
        'ulp-mlperf',
        'datasets')

    print('')
    print('Path to EEMBC runner datasets: {}'.format(datasets_path))

    if not os.path.isdir(datasets_path):
        os.makedirs(datasets_path)

    env['CM_EEMBC_ENERGY_RUNNER_DATASETS'] = datasets_path

    print('')
    print('Copying datasets to EEMBC user space ...')

    shutil.copytree(datasets_src_path, datasets_path, dirs_exist_ok=True)

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os
import subprocess


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    submission_dir = env.get("CM_MLPERF_SUBMISSION_DIR", "")

    version = env.get('CM_MLPERF_SUBMISSION_CHECKER_VERSION', 'v3.1')

    if submission_dir == "":
        return {'return': 1, 'error': 'Please set CM_MLPERF_SUBMISSION_DIR'}

    submitter = env.get("CM_MLPERF_SUBMITTER", "")  # "default")
    if ' ' in submitter:
        return {
            'return': 1, 'error': 'CM_MLPERF_SUBMITTER cannot contain a space. Please provide a name without space using --submitter input. Given value: {}'.format(submitter)}

    submission_checker_file = os.path.join(
        env['CM_MLPERF_LOGGING_REPO_PATH'],
        "scripts",
        "verify_for_" + version + "_training.sh")

    extra_args = ' ' + env.get('CM_MLPERF_SUBMISSION_CHECKER_EXTRA_ARGS', '')

    CMD = submission_checker_file + " " + submission_dir

    env['CM_RUN_CMD'] = CMD

    return {'return': 0}


def postprocess(i):

    env = i['env']
    if env.get('CM_TAR_SUBMISSION_DIR'):
        env['CM_TAR_INPUT_DIR'] = env.get('CM_MLPERF_SUBMISSION_DIR', '$HOME')

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    automation = i['automation']
    run_script_input = i['run_script_input']

    recursion_spaces = i['recursion_spaces']

    if env.get('CM_CONDA_ENV_NAME', '') == '':
        return {'return': 1, 'error': 'Please use "_name.<conda env name>" variation'}

    return {'return': 0}


def postprocess(i):
    env = i['env']

    conda_prefix = os.getcwd()
    env['CONDA_PREFIX'] = conda_prefix
    env['CM_CONDA_PREFIX'] = conda_prefix
    env['CM_CONDA_BIN_PATH'] = os.path.join(conda_prefix, "bin")
    env['CM_CONDA_LIB_PATH'] = os.path.join(conda_prefix, "lib")

    env['+PATH'] = [env['CM_CONDA_BIN_PATH']]
    env['+LD_LIBRARY_PATH'] = [env['CM_CONDA_LIB_PATH']]

    return {'return': 0}



#!/usr/bin/env python3

# Extended by Grigori Fursin to support MLCommons CM workflow automation
# language

import os
import onnxruntime as rt
import numpy as np
import time
import json

from PIL import Image

model_path = os.environ['CK_ENV_ONNX_MODEL_ONNX_FILEPATH']
input_layer_name = os.environ['CK_ENV_ONNX_MODEL_INPUT_LAYER_NAME']
output_layer_name = os.environ['CK_ENV_ONNX_MODEL_OUTPUT_LAYER_NAME']
normalize_data_bool = os.getenv(
    'CK_ENV_ONNX_MODEL_NORMALIZE_DATA', '0') in (
        'YES', 'yes', 'ON', 'on', '1')
subtract_mean_bool = os.getenv(
    'CK_ENV_ONNX_MODEL_SUBTRACT_MEAN', '0') in (
        'YES', 'yes', 'ON', 'on', '1')
given_channel_means = os.getenv('ML_MODEL_GIVEN_CHANNEL_MEANS', '')
if given_channel_means:
    given_channel_means = np.array(
        given_channel_means.split(' '),
        dtype=np.float32)

imagenet_path = os.environ['CK_ENV_DATASET_IMAGENET_VAL']
labels_path = os.environ['CK_CAFFE_IMAGENET_SYNSET_WORDS_TXT']
data_layout = os.environ['ML_MODEL_DATA_LAYOUT']
batch_size = int(os.environ['CK_BATCH_SIZE'])
batch_count = int(os.environ['CK_BATCH_COUNT'])
CPU_THREADS = int(os.getenv('CK_HOST_CPU_NUMBER_OF_PROCESSORS', 0))


def load_labels(labels_filepath):
    my_labels = []
    input_file = open(labels_filepath, 'r')
    for l in input_file:
        my_labels.append(l.strip())
    return my_labels


def load_and_resize_image(image_filepath, height, width):
    # sic! The order of dimensions in resize is (W,H)
    pillow_img = Image.open(image_filepath).resize((width, height))

    # Grigori fixed below
    # input_data = np.float32(pillow_img)
    input_data = np.asarray(pillow_img)
    input_data = np.asarray(input_data, np.float32)

    # Normalize
    if normalize_data_bool:
        input_data = input_data / 127.5 - 1.0

    # Subtract mean value
    if subtract_mean_bool:
        if len(given_channel_means):
            input_data -= given_channel_means
        else:
            input_data -= np.mean(input_data)

#    print(np.array(pillow_img).shape)
    nhwc_data = np.expand_dims(input_data, axis=0)

    if data_layout == 'NHWC':
        # print(nhwc_data.shape)
        return nhwc_data
    else:
        nchw_data = nhwc_data.transpose(0, 3, 1, 2)
        # print(nchw_data.shape)
        return nchw_data


def load_a_batch(batch_filenames):
    unconcatenated_batch_data = []
    for image_filename in batch_filenames:
        image_filepath = image_filename
        nchw_data = load_and_resize_image(image_filepath, height, width)
        unconcatenated_batch_data.append(nchw_data)
    batch_data = np.concatenate(unconcatenated_batch_data, axis=0)

    return batch_data


# print("Device: " + rt.get_device())
sess_options = rt.SessionOptions()

if CPU_THREADS > 0:
    sess_options.enable_sequential_execution = False
    sess_options.session_thread_pool_size = CPU_THREADS

if len(rt.get_all_providers()) > 1 and os.environ.get(
        "USE_CUDA", "yes").lower() not in ["0", "false", "off", "no"]:
    # Currently considering only CUDAExecutionProvider
    sess = rt.InferenceSession(
        model_path,
        sess_options,
        providers=['CUDAExecutionProvider'])
else:
    sess = rt.InferenceSession(
        model_path,
        sess_options,
        providers=["CPUExecutionProvider"])

# FIXME: check that input_layer_name belongs to this list
input_layer_names = [x.name for x in sess.get_inputs()]
input_layer_name = input_layer_name or input_layer_names[0]

# FIXME: check that output_layer_name belongs to this list
output_layer_names = [x.name for x in sess.get_outputs()]
output_layer_name = output_layer_name or output_layer_names[0]

model_input_shape = sess.get_inputs()[0].shape
model_classes = sess.get_outputs()[1].shape[1]
labels = load_labels(labels_path)
# 1 means the labels represent classes 1..1000 and the background class 0
# has to be skipped
bg_class_offset = model_classes - len(labels)

if data_layout == 'NHWC':
    (samples, height, width, channels) = model_input_shape
else:
    (samples, channels, height, width) = model_input_shape

print("")
print("Data layout: {}".format(data_layout))
print("Input layers: {}".format([str(x) for x in sess.get_inputs()]))
print("Output layers: {}".format([str(x) for x in sess.get_outputs()]))
print("Input layer name: " + input_layer_name)
print("Expected input shape: {}".format(model_input_shape))
print("Output layer name: " + output_layer_name)
print("Data normalization: {}".format(normalize_data_bool))
print("Subtract mean: {}".format(subtract_mean_bool))
print('Per-channel means to subtract: {}'.format(given_channel_means))
print("Background/unlabelled classes to skip: {}".format(bg_class_offset))
print("")

starting_index = 1

start_time = time.time()

for batch_idx in range(batch_count):
    print('')
    print("Batch {}/{}:".format(batch_idx + 1, batch_count))

    batch_filenames = [
        imagenet_path +
        '/' +
        "ILSVRC2012_val_00000{:03d}.JPEG".format(
            starting_index +
            batch_idx *
            batch_size +
            i) for i in range(batch_size)]

    # Grigori: trick to test models:
    if os.environ.get('CM_IMAGE', '') != '':
        batch_filenames = [os.environ['CM_IMAGE']]

    batch_data = load_a_batch(batch_filenames)
    # print(batch_data.shape)

    batch_predictions = sess.run(
        [output_layer_name], {
            input_layer_name: batch_data})[0]

    cm_status = {'classifications': []}

    print('')
    top_classification = ''
    for in_batch_idx in range(batch_size):
        # skipping the background class on the left (if present)
        softmax_vector = batch_predictions[in_batch_idx][bg_class_offset:]
        top5_indices = list(reversed(softmax_vector.argsort()))[:5]

        print(' * ' + batch_filenames[in_batch_idx] + ' :')

        for class_idx in top5_indices:
            if top_classification == '':
                top_classification = labels[class_idx]

            print(
                "\t{}\t{}\t{}".format(
                    class_idx,
                    softmax_vector[class_idx],
                    labels[class_idx]))

            cm_status['classifications'].append({'class_idx': int(class_idx),
                                                 'softmax': float(softmax_vector[class_idx]),
                                                 'label': labels[class_idx]})

    print('')
    print('Top classification: {}'.format(top_classification))
    cm_status['top_classification'] = top_classification

avg_time = (time.time() - start_time) / batch_count
cm_status['avg_time'] = avg_time

# Record cm_status to embedded it into CM workflows
with open('tmp-run-state.json', 'w') as cm_file:
    cm_file.write(json.dumps(
        {'cm_app_image_classification_onnx_py': cm_status}, sort_keys=True, indent=2))



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import cmind as cm


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    meta = i['meta']
    automation = i['automation']
    run_script_input = i['run_script_input']

    version_string = env.get('CM_TMP_PIP_VERSION_STRING', '').strip()
    package_name = env['CM_CONDA_PKG_NAME'].strip()

    install_cmd = env['CM_CONDA_BIN_WITH_PATH'] + " install -y "
    if env.get('CM_CONDA_PKG_SRC', '') != '':
        install_cmd += " -c " + env['CM_CONDA_PKG_SRC'] + " "

    install_cmd += package_name
    install_cmd += version_string

    env['CM_CONDA_PKG_INSTALL_CMD'] = install_cmd

    return {'return': 0}


def detect_version(i):

    # TBD
    print(i['recursion_spaces'] + '      Detected version: {}'.format(version))

    return {'return': 0, 'version': version}


def postprocess(i):

    env = i['env']
    version = env.get('CM_VERSION', '')

    if env['CM_CONDA_PKG_NAME'] == "python":
        env['CM_PYTHON_BIN_WITH_PATH'] = os.path.join(
            os.path.dirname(env['CM_CONDA_BIN_WITH_PATH']), "python")

    return {'return': 0, 'version': version}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):
    os_info = i['os_info']
    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}

    return {'return': 0}


def postprocess(i):

    env = i['env']
    installed_path = os.path.join(os.getcwd(), 'bin')
    env['CM_TERRAFORM_INSTALLED_PATH'] = installed_path
    env['CM_TERRAFORM_BIN_WITH_PATH'] = os.path.join(
        installed_path, "terraform")
    env['+PATH'] = [installed_path]

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    env = i['env']

    return {'return': 0}


def postprocess(i):
    env = i['env']
    if env.get('CM_DATASET_CALIBRATION', '') == "no":
        env['CM_DATASET_PATH_ROOT'] = env['CM_DATASET_OPENORCA_PATH']
        env['CM_DATASET_PATH'] = env['CM_DATASET_OPENORCA_PATH']
        env['CM_DATASET_OPENORCA_PARQUET'] = os.path.join(
            env['CM_DATASET_OPENORCA_PATH'], '1M-GPT4-Augmented.parquet')
    else:
        env['CM_CALIBRATION_DATASET_PATH'] = os.path.join(
            os.getcwd(), 'install', 'calibration', 'data')

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import hashlib


def preprocess(i):

    variation_tags = i.get('variation_tags', [])

    os_info = i['os_info']

    windows = os_info['platform'] == 'windows'

#    xsep = '^&^&' if windows else '&&'
    xsep = '&&'
    q = '"' if os_info['platform'] == 'windows' else "'"

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    filename = env.get('CM_EXTRACT_FILEPATH', '')
    if filename == '':
        return {
            'return': 1, 'error': 'Extract with no download requested and CM_EXTRACT_FILEPATH is not set'}

    if windows:
        filename = filename.replace("%", "%%")

    env['CM_EXTRACT_FILENAME'] = filename

    # Check if extract to some path outside CM cache (to reuse large files
    # later if cache is cleaned)
    extract_path = env.get('CM_EXTRACT_PATH', '')
    if extract_path != '':
        if not os.path.exists(extract_path):
            os.makedirs(extract_path, exist_ok=True)

        os.chdir(extract_path)

    # By default remove archive after extraction
    remove_extracted = False if env.get(
        'CM_EXTRACT_REMOVE_EXTRACTED',
        '').lower() == 'no' else True

    if filename.endswith(".zip") or filename.endswith(".pth"):
        env['CM_EXTRACT_TOOL'] = "unzip"
    elif filename.endswith(".tar.gz"):
        if windows:
            x = '"' if ' ' in filename else ''
            env['CM_EXTRACT_CMD0'] = 'gzip -d ' + x + filename + x
            filename = filename[:-3]  # leave only .tar
            env['CM_EXTRACT_TOOL_OPTIONS'] = ' -xvf'
            env['CM_EXTRACT_TOOL'] = 'tar '
        elif os_info['platform'] == 'darwin':
            env['CM_EXTRACT_TOOL_OPTIONS'] = ' -xvzf '
            env['CM_EXTRACT_TOOL'] = 'tar '
        else:
            env['CM_EXTRACT_TOOL_OPTIONS'] = ' --skip-old-files -xvzf '
            env['CM_EXTRACT_TOOL'] = 'tar '
    elif filename.endswith(".tar.xz"):
        if windows:
            x = '"' if ' ' in filename else ''
            env['CM_EXTRACT_CMD0'] = 'xz -d ' + x + filename + x
            filename = filename[:-3]  # leave only .tar
            env['CM_EXTRACT_TOOL_OPTIONS'] = ' -xvf'
            env['CM_EXTRACT_TOOL'] = 'tar '
        else:
            env['CM_EXTRACT_TOOL_OPTIONS'] = ' -xvJf'
            env['CM_EXTRACT_TOOL'] = 'tar '
    elif filename.endswith(".tar"):
        env['CM_EXTRACT_TOOL_OPTIONS'] = ' -xvf'
        env['CM_EXTRACT_TOOL'] = 'tar '
    elif filename.endswith(".gz"):
        # Check target filename
        extracted_filename = env.get('CM_EXTRACT_EXTRACTED_FILENAME', '')
        if extracted_filename == '':
            extracted_filename = os.path.basename(filename)[:-3]
            env['CM_EXTRACT_EXTRACTED_FILENAME'] = extracted_filename

        x = '-c' if windows else '-k'
        env['CM_EXTRACT_TOOL_OPTIONS'] = ' -d ' + \
            (x + ' ' if not remove_extracted else '') + \
            ' > ' + q + extracted_filename + q + ' < '

        env['CM_EXTRACT_TOOL'] = 'gzip '
    elif env.get('CM_EXTRACT_UNZIP', '') == 'yes':
        env['CM_EXTRACT_TOOL'] = 'unzip '
    elif env.get('CM_EXTRACT_UNTAR', '') == 'yes':
        env['CM_EXTRACT_TOOL_OPTIONS'] = ' -xvf'
        env['CM_EXTRACT_TOOL'] = 'tar '
    elif env.get('CM_EXTRACT_GZIP', '') == 'yes':
        env['CM_EXTRACT_CMD'] = 'gzip '
        env['CM_EXTRACT_TOOL_OPTIONS'] = ' -d ' + \
            ('-k ' if not remove_extracted else '')
    else:
        return {'return': 1,
                'error': 'Neither CM_EXTRACT_UNZIP nor CM_EXTRACT_UNTAR is yes'}

    env['CM_EXTRACT_PRE_CMD'] = ''

    extract_to_folder = env.get('CM_EXTRACT_TO_FOLDER', '')

    # Check if extract to additional folder in the current directory (or external path)
    # to avoid messing up other files and keep clean directory structure
    # particularly if archive has many sub-directories and files
    if extract_to_folder != '':
        if 'tar ' in env['CM_EXTRACT_TOOL']:
            x = '' if windows else '-p'
            y = '"' if ' ' in extract_to_folder else ''

            # env['CM_EXTRACT_TOOL_OPTIONS'] = ' --one-top-level='+ env['CM_EXTRACT_TO_FOLDER'] + env.get('CM_EXTRACT_TOOL_OPTIONS', '')
            env['CM_EXTRACT_TOOL_OPTIONS'] = ' -C ' + y + extract_to_folder + \
                y + ' ' + env.get('CM_EXTRACT_TOOL_OPTIONS', '')
            env['CM_EXTRACT_PRE_CMD'] = 'mkdir ' + x + ' ' + \
                y + extract_to_folder + y + ' ' + xsep + ' '
            env['CM_EXTRACT_EXTRACTED_FILENAME'] = extract_to_folder

        elif 'unzip' in env['CM_EXTRACT_TOOL']:
            env['CM_EXTRACT_TOOL_OPTIONS'] = ' -d ' + q + extract_to_folder + q
            env['CM_EXTRACT_EXTRACTED_FILENAME'] = extract_to_folder

    x = '"' if ' ' in filename else ''
    env['CM_EXTRACT_CMD'] = env['CM_EXTRACT_PRE_CMD'] + env['CM_EXTRACT_TOOL'] + ' ' + \
        env.get('CM_EXTRACT_TOOL_EXTRA_OPTIONS', '') + \
        ' ' + env.get('CM_EXTRACT_TOOL_OPTIONS', '') + ' ' + x + filename + x

    print('')
    print('Current directory: {}'.format(os.getcwd()))
    print('Command line: "{}"'.format(env['CM_EXTRACT_CMD']))
    print('')

    final_file = env.get('CM_EXTRACT_EXTRACTED_FILENAME', '')

    if final_file != '':
        if env.get('CM_EXTRACT_EXTRACTED_CHECKSUM_FILE', '') != '':
            env['CM_EXTRACT_EXTRACTED_CHECKSUM_CMD'] = f"cd {q}{final_file}{q} {xsep}  md5sum -c {q}{env['CM_EXTRACT_EXTRACTED_CHECKSUM_FILE']}{q}"
        elif env.get('CM_EXTRACT_EXTRACTED_CHECKSUM', '') != '':
            x = '*' if os_info['platform'] == 'windows' else ''
            env['CM_EXTRACT_EXTRACTED_CHECKSUM_CMD'] = "echo {} {}{q}{}{q} | md5sum -c".format(
                env.get('CM_EXTRACT_EXTRACTED_CHECKSUM'), x, env['CM_EXTRACT_EXTRACTED_FILENAME'])
        else:
            env['CM_EXTRACT_EXTRACTED_CHECKSUM_CMD'] = ""
    else:
        env['CM_EXTRACT_EXTRACTED_CHECKSUM_CMD'] = ""

# Not needed - can be simpler with cmd /c {empty}
#    if os_info['platform'] == 'windows':
#        # Check that if empty CMD, should add ""
#        for x in ['CM_EXTRACT_CMD', 'CM_EXTRACT_EXTRACTED_CHECKSUM_CMD']:
#            env[x+'_USED']='YES' if env.get(x,'')!='' else 'NO'

    # If force cache, add filepath to tag unless _path is used ...
    path_tag = 'path.' + filename

    add_extra_cache_tags = []
    if path_tag not in variation_tags:
        add_extra_cache_tags.append(path_tag)

    return {'return': 0, 'add_extra_cache_tags': add_extra_cache_tags}


def postprocess(i):

    automation = i['automation']

    env = i['env']

    extract_to_folder = env.get('CM_EXTRACT_TO_FOLDER', '')
    extract_path = env.get('CM_EXTRACT_PATH', '')

    extracted_file = env.get('CM_EXTRACT_EXTRACTED_FILENAME', '')

    # Preparing filepath
    #   Can be either full extracted filename (such as model) or folder

    if extracted_file != '':
        filename = os.path.basename(extracted_file)

# We do not use this env variable anymore
#        folderpath = env.get('CM_EXTRACT_EXTRACT_TO_PATH', '')
        folderpath = extract_path if extract_path != '' else os.getcwd()

        filepath = os.path.join(folderpath, filename)
    else:

        filepath = os.getcwd()  # Extracted to the root cache folder

    if not os.path.exists(filepath):
        return {
            'return': 1, 'error': 'Path {} was not created or doesn\'t exist'.format(filepath)}
# return {'return':1, 'error': 'CM_EXTRACT_EXTRACTED_FILENAME and
# CM_EXTRACT_TO_FOLDER are not set'}

    env['CM_EXTRACT_EXTRACTED_PATH'] = filepath

    # Set external environment variable with the final path
    if env.get('CM_EXTRACT_FINAL_ENV_NAME', '') != '':
        env[env['CM_EXTRACT_FINAL_ENV_NAME']] = filepath

    # Detect if this file will be deleted or moved
    env['CM_GET_DEPENDENT_CACHED_PATH'] = filepath

    # Check if need to remove archive after extraction
    if env.get('CM_EXTRACT_REMOVE_EXTRACTED', '').lower() != 'no':
        archive_filepath = env.get('CM_EXTRACT_FILEPATH', '')
        if archive_filepath != '' and os.path.isfile(archive_filepath):
            os.remove(archive_filepath)

    # Since may change directory, check if need to clean some temporal files
    automation.clean_some_tmp_files({'env': env})

    return {'return': 0}



# Developer(s): Grigori Fursin

import cmind
import os
import misc

import streamlit.components.v1 as components

import streamlit as st

import json

announcement = 'Under development - please get in touch via [Discord](https://discord.gg/JjWNWXKxwT) for more details ...'

badges = {
    'functional': {'url': 'https://cTuning.org/images/artifacts_evaluated_functional_v1_1_small.png'},
    'reproduced': {'url': 'https://cTuning.org/images/results_reproduced_v1_1_small.png'},
    'support_docker': {'url': 'https://cTuning.org/images/docker_logo2_small.png'},
    'support_cm': {'url': 'https://cTuning.org/images/logo-ck-single-tr4.png'}
}


def main():
    params = misc.get_params(st)

    # Set title
    st.title('Reproducibility studies')

    st.markdown(announcement)

    return page(st, params)


def page(st, params, action=''):

    end_html = ''


#    st.markdown('----')

    self_url = misc.make_url('', key='', action='reproduce', md=False)
    url_benchmarks = misc.make_url('', key='', action='howtorun', md=False)
    url_challenges = misc.make_url('', key='', action='challenges', md=False)

    # Some info
    x = '''
         <i>
         <small>
         [Under development] This is a new project to reproduce <a href="{}">modular benchmarks</a>
         across different models, data sets, software and hardware
         via <a href="{}">open challenges</a>
         based on the <a href="https://cTuning.org/ae">ACM/cTuning reproducibility methodology and badges</a>
         and <a href="https://sites.google.com/g.harvard.edu/mlperf-bench-hpca24/home">automatically compose
         High-Performance and Cost-Efficient AI Systems with MLCommons' Collective Mind and MLPerf</a>.
         Note that this is a <a href="https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md">collaborative engineering effort</a>
         - please report encountered issues and provide feedback
         <a href="https://github.com/mlcommons/ck/issues">here</a>
         and get in touch via <a href="https://discord.gg/JjWNWXKxwT">Discord</a>!
         </small>
         </i>
          <br>
          <br>
        '''.format(url_benchmarks, url_challenges)

    st.write(x, unsafe_allow_html=True)

    return {'return': 0}


#    st.markdown(announcement)

    # Check if test is selected
    test_uid = ''
    x = params.get('test_uid', [''])
    if len(x) > 0 and x[0] != '':
        test_uid = x[0].strip()

    ##########################################################################
    # Select target hardware
    compute_uid = ''
    compute_meta = {}
    compute_selection = []

    if test_uid == '':
        x = params.get('compute_uid', [''])
        if len(x) > 0 and x[0] != '':
            compute_uid = x[0].strip()

    ii = {'action': 'load_cfg',
          'automation': 'utils',
          'tags': 'benchmark,compute',
          'skip_files': False}

    if compute_uid != '':
        ii['prune'] = {'uid': compute_uid}

    r = cmind.access(ii)
    if r['return'] > 0:
        return r
    compute_selection = r['selection']

    if test_uid == '':
        r = misc.make_selection(
            st,
            r['selection'],
            'compute',
            'target hardware',
            compute_uid)
        if r['return'] > 0:
            return r
        compute_meta = r['meta']
        compute_uid = compute_meta.get('uid', '')

    ##########################################################################
    # Select benchmark
    bench_meta = {}

    bench_name = ''
    x = params.get('bench_name', [''])
    if len(x) > 0 and x[0] != '':
        bench_name = x[0].strip()

    if test_uid == '':
        ii = {'action': 'load_cfg',
              'automation': 'utils',
              'tags': 'benchmark,run',
              'skip_files': True}

        if bench_name != '':
            ii['artifact'] = bench_name

        r = cmind.access(ii)
        if r['return'] > 0:
            return r

        # Prune by supported compute
        selection = r['selection']
        pruned_selection = []

        if compute_uid == '':
            pruned_selection = selection
        else:
            for s in selection:
                add = True

                if compute_uid in s.get('supported_compute', []):
                    pruned_selection.append(s)

        r = misc.make_selection(
            st,
            pruned_selection,
            'benchmark',
            'benchmark',
            bench_name)
        if r['return'] > 0:
            return r

        bench_meta = r['meta']

    ##########################################################################
    # Select tests
    if test_uid == '' and compute_uid == '' and len(bench_meta) == 0:
        st.markdown('*Please prune search by device and/or benchmark ...*')

    else:
        ii = {'action': 'load_cfg',
              'automation': 'utils',
              'tags': 'benchmark,run',
              'key': 'run-',
              'key_end': ['-meta.json', '-meta.yaml'],
              'skip_files': False}

        if len(bench_meta) > 0 or bench_name != '':
            if len(bench_meta) > 0:
                ii['artifact'] = bench_meta['uid']
            else:
                ii['artifact'] = bench_name
        elif compute_uid != '':
            ii['prune'] = {'meta_key': 'supported_compute',
                           'meta_key_uid': compute_uid}

        if compute_uid != '':
            if 'prune' not in ii:
                ii['prune'] = {}
            ii['prune']['key'] = 'compute_uid'
            ii['prune']['key_uid'] = compute_uid

        if test_uid != '':
            if 'prune' not in ii:
                ii['prune'] = {}
            ii['prune']['uid'] = test_uid

        r = cmind.access(ii)
        if r['return'] > 0:
            return r

        # Prune by supported compute
        selection = r['selection']

        if len(selection) == 0:
            st.markdown('*WARNING: No tests found!*')
        else:
            if len(selection) == 1:
                ###############################################################
                # Show individual test
                s = selection[0]

                full_path = s['full_path']
                test_uid = s['uid']

                st.markdown('---')
                st.markdown('**Test {}**'.format(test_uid))

                # Check badges
                x = ''

                for b in badges:
                    if s.get(b, False) or b == 'support_cm':
                        x += '<a href="http://cTuning.org/ae" target="_blank"><img src="{}" height="64"></a>\n'.format(
                            badges[b]['url'])

                if x != '':
                    st.write(x, unsafe_allow_html=True)

                # Check benchmark
                bench_uid = s.get('bench_uid', '')
                if bench_uid != '':
                    url_bench = url_benchmarks + '&bench_uid=' + bench_uid
                    st.markdown(
                        '[Link to benchmark GUI]({})'.format(url_bench))

                # Check notes
                test_md = full_path[:-10] + '.md'
                if os.path.isfile(test_md):

                    r = cmind.utils.load_txt(test_md)
                    if r['return'] > 0:
                        return r

                    x = r['string']

                    if x != '':
                        st.markdown('**Notes:**')
                        st.markdown(x)

                inp = {}
                input_file = full_path[:-10] + '-input'
                r = cmind.utils.load_yaml_and_json(input_file)
                if r['return'] == 0:
                    inp = r['meta']

                out = {}
                output_file = full_path[:-10] + '-output'
                r = cmind.utils.load_yaml_and_json(output_file)
                if r['return'] == 0:
                    out = r['meta']

                cmd = inp.get('cmd', [])
                if len(cmd) > 0:
                    xcmd = ' \\\n   '.join(cmd)

                    st.markdown("""
 **CM command line:**
 ```bash
 cm run script {}
 ```
                                """.format(xcmd))

                st.markdown("""
**CM input dictionary:**
```json
{}
```
                           """.format(json.dumps(inp, indent=2)))

                st.markdown("""
**CM input dictionary:**
```json
{}
```
                               """.format(json.dumps(out, indent=2)))

                st.markdown("""

**Test meta:**
```json
{}
```
                            """.format(json.dumps(s, indent=2)))

            else:
                ###############################################################
                # Show tables
                import pandas as pd
                import numpy as np

                html = ''

                all_data = []

                # TBD: should be taken from a given benchmark
                dimensions = []

                if len(bench_meta) > 0:
                    dimensions = bench_meta.get('view_dimensions', [])

                dimension_values = {}
                dimension_keys = []

                if len(dimensions) == 0:
                    keys = [('test', 'CM test', 400, 'leftAligned')]
                else:
                    keys = [('test', 'CM test', 50, 'leftAligned')]

                    for k in dimensions:
                        key = k[0]

                        keys.append((k[0], k[1], 100, 'leftAligned'))

                        dimension_values[key] = []
                        dimension_keys.append(key)

                    # If dimensions, sort by dimensions
                    for d in list(reversed(dimension_keys)):
                        selection = sorted(
                            selection, key=lambda x: misc.get_with_complex_key_safe(
                                selection, d))

                keys += [
                    ('functional', '<a href="https://github.com/mlcommons/ck/blob/master/docs/artifact-evaluation/submission.md">Functional</a>', 80, ''),
                    (
                        'reproduced',
                        '<a href="https://github.com/mlcommons/ck/blob/master/docs/artifact-evaluation/submission.md">Reproduced</a>',
                        80,
                        ''),
                    ('support_docker', 'Support Docker', 80, ''),
                    (
                        'support_cm',
                        '<a href="https://github.com/mlcommons/ck">Has unified CM interface</a>',
                        80,
                        ''),
                    ('notes', 'Notes', 200, 'lefAligned'),
                ]

                j = 0

                for s in selection:

                    row = {}

                    full_path = s['full_path']
                    test_uid = s['uid']

                    uid = s['uid']

                    url_test = misc.make_url(
                        uid, key='test_uid', action='reproduce', md=False)

                    bench_meta = s['main_meta']

                    inp = {}
                    input_file = full_path[:-10] + '-input'
                    r = cmind.utils.load_yaml_and_json(input_file)
                    if r['return'] == 0:
                        inp = r['meta']

                    out = {}
                    output_file = full_path[:-10] + '-output'
                    r = cmind.utils.load_yaml_and_json(output_file)
                    if r['return'] == 0:
                        out = r['meta']

                    row_meta = {'dict': s,
                                'input': inp,
                                'output': out}

                    if len(dimensions) == 0:
                        row['test'] = '<a href="{}" target="_blank">{}</a>'.format(
                            url_test, uid)
                    else:
                        row['test'] = '<a href="{}" target="_blank">View</a>'.format(
                            url_test)
                        for k in dimensions:
                            kk = k[0]

                            v = misc.get_with_complex_key_safe(row_meta, kk)

                            if len(k) > 2 and k[2] == 'tick':
                                if v is not None and v != '':
                                    v = '✅'

                            row[kk] = str(v)

                    # Check ACM/IEEE functional badge
                    url = ''

                    x = ''
                    if s.get('functional', False):
                        x = '<center><a href="{}" target="_blank"><img src="{}" height="32"></a></center>'.format(
                            url, badges['functional']['url'])
                    row['functional'] = x

                    # Check ACM/IEEE reproduced badge
                    x = ''
                    if s.get('reproduced', False):
                        x = '<center><a href="{}" target="_blank"><img src="{}" height="32"></a></center>'.format(
                            url, badges['reproduced']['url'])
                    row['reproduced'] = x

                    # Check Docker
                    x = ''
                    if s.get('support_docker', False):
                        x = '<center><a href="{}" target="_blank"><img src="{}" height="32"></a></center>'.format(
                            url, badges['support_docker']['url'])
                    row['support_docker'] = x

                    x = ''
                    bench_uid = s.get('bench_uid', '')
                    if bench_uid != '':
                        url_bench = url_benchmarks + '&bench_uid=' + bench_uid
                        x = '<center><a href="{}" target="_blank"><img src="{}" height="32"></a></center>'.format(
                            url_bench, badges['support_cm']['url'])
                    row['support_cm'] = x

                    # Check misc notes
                    row['notes'] = '<small>' + s.get('notes', '') + '</small>'

                    # Finish row
                    all_data.append(row)

                # Visualize table
                pd_keys = [v[0] for v in keys]
                pd_key_names = [v[1] for v in keys]

                pd_all_data = []
                for row in sorted(
                        all_data, key=lambda row: (row.get('x1', 0))):
                    pd_row = []
                    for k in pd_keys:
                        pd_row.append(row.get(k))
                    pd_all_data.append(pd_row)

                df = pd.DataFrame(pd_all_data, columns=pd_key_names)

                df.index += 1

                html = df.to_html(escape=False, justify='left')
                st.write(html, unsafe_allow_html=True)

    if bench_name != '':
        self_url += '&bench_name=' + bench_name
    if test_uid != '':
        self_url += '&test_uid=' + test_uid
    elif compute_uid != '':
        self_url += '&compute_uid=' + compute_uid

    end_html = '<center><small><i><a href="{}">Self link</a></i></small></center>'.format(
        self_url)

    return {'return': 0, 'end_html': end_html}
